
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "zend_exceptions.h"

#include <senna.h>
#include "php_senna.h"
#include "php_senna_in.h"

#include "senna_index.h"
#include "senna_records.h"
#include "senna_snippet.h"


/* If you declare any globals in php_senna.h uncomment this: */
ZEND_DECLARE_MODULE_GLOBALS(senna)
static PHP_GINIT_FUNCTION(senna);

/* True global resources - no need for thread safety here
static int le_senna;
*/

zend_object_handlers SENNA_HANDLER(Senna);
PHPAPI zend_class_entry  *SENNA_CE(Senna);

/* SPL΂gAȂΎŒ` */
#ifdef PHP_SENNA_CAN_DEPENDS_ON_SPL
extern zend_class_entry *SPL_CE(RuntimeException);
#else
static zend_class_entry *SPL_CE(RuntimeException); 
zend_class_entry *SPL_CE(OutOfBoundsException);
zend_class_entry *SPL_CE(UnexpectedValueException);
zend_class_entry *SPL_CE(UnexpectedValueException);

zend_class_entry *SPL_CE(BadMethodCallException);
zend_class_entry *SPL_CE(InvalidArgumentException);
#endif

PHPAPI zend_class_entry *SENNA_CE(SennaException);

/* {{{ php_senna_free() free for Senna object */
void php_senna_free(void *object TSRMLS_DC) {
	php_senna_t *senna = (php_senna_t *) object;
	
	zend_object_std_dtor(&senna->std TSRMLS_CC);

	efree(object);
} /* }}} */

/* {{{ php_senna_new() allocate for Senna object */
static zend_object_value php_senna_new(zend_class_entry *class_type TSRMLS_DC) {
	zend_object_value value;
	php_senna_t *senna;
	zval *tmp;
	
	senna = ecalloc(1, sizeof(*senna));

	value.handlers = &SENNA_HANDLER(Senna);
	value.handle = zend_objects_store_put(senna, (zend_objects_store_dtor_t) zend_objects_destroy_object, (zend_objects_free_object_storage_t) php_senna_free, NULL TSRMLS_CC);
	
	zend_object_std_init(&senna->std, class_type TSRMLS_CC);
	zend_hash_copy(senna->std.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));

	return value;
} /* }}} */


/* {{{ senna_functions[]
 */
zend_function_entry senna_functions[] = {
/*
	PHP_ME(Senna, init, NULL, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
	PHP_ME(Senna, fin, NULL, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
 */
	{NULL, NULL, NULL}	/* Must be the last line in senna_functions[] */
};
/* }}} */

/* {{{ senna_deps[] */
/* Dependancies */
#if ZEND_MODULE_API_NO >= 20050922
static zend_module_dep senna_deps[] = {
#ifdef PHP_SENNA_CAN_DEPENDS_ON_SPL
	ZEND_MOD_REQUIRED("spl")
#endif
	{NULL, NULL, NULL}
};
#endif
/* }}} */

/* {{{ senna_module_entry
 */
zend_module_entry senna_module_entry = {
#if ZEND_MODULE_API_NO >= 20050922
	STANDARD_MODULE_HEADER_EX, NULL,
	senna_deps,
#elif ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"senna",
	senna_functions,
	PHP_MINIT(senna),
	PHP_MSHUTDOWN(senna),
	PHP_RINIT(senna),		/* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(senna),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(senna),
	"0.8.0",
	PHP_MODULE_GLOBALS(senna),
	PHP_GINIT(senna),
	NULL,
	NULL,
	STANDARD_MODULE_PROPERTIES_EX
};
/* }}} */

#ifdef COMPILE_DL_SENNA
ZEND_GET_MODULE(senna)
#endif

/* {{{ PHP_INI
 */
/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("senna.global_value",      "42", PHP_INI_ALL, OnUpdateLong, global_value, zend_senna_globals, senna_globals)
    STD_PHP_INI_ENTRY("senna.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_senna_globals, senna_globals)
PHP_INI_END()
*/
/* }}} */

/* {{{ php_senna_init_globals
 */
/* Uncomment this function if you have INI entries
static void php_senna_init_globals(zend_senna_globals *senna_globals)
{
	senna_globals->global_value = 0;
	senna_globals->global_string = NULL;
}
*/
/* }}} */

PHP_METHOD(Senna, __construct) {
}

zend_function_entry php_senna_functions[] = {
	PHP_ME(Senna, __construct, NULL, ZEND_ACC_PRIVATE | ZEND_ACC_CTOR)
	{NULL, NULL, NULL}
};

/* {{{ PHP_GINIT_FUNCTION */
static PHP_GINIT_FUNCTION(senna) {
	/* Apache̓쒆ɃCuXV邩Ȃ̂RINITŏ */
	/*
	sen_info(
		&senna_globals->version,
		NULL,
		NULL,
		&senna_globals->default_encoding,
		NULL,
		NULL
	);
	*/
}
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(senna) {
	/* If you have INI entries, uncomment these lines 
	REGISTER_INI_ENTRIES();
	*/

#ifndef PHP_SENNA_CAN_DEPENDS_ON_SPL
	/* SPLgȂꍇ́AftHgExceptionNXg */
#	if (PHP_MAJOR_VERSION == 5) && (PHP_MINOR_VERSION < 2)
		SPL_CE(RuntimeException) = zend_exception_get_default();
		SPL_CE(OutOfBoundsException) = zend_exception_get_default();
		SPL_CE(UnexpectedValueException) = zend_exception_get_default();
		SPL_CE(BadMethodCallException) = zend_exception_get_default();
		SPL_CE(InvalidArgumentException) = zend_exception_get_default();
#	else
		SPL_CE(RuntimeException) = zend_exception_get_default(TSRMLS_C);
		SPL_CE(OutOfBoundsException) = zend_exception_get_default(TSRMLS_C);
		SPL_CE(UnexpectedValueException) = zend_exception_get_default(TSRMLS_C);
		SPL_CE(BadMethodCallException) = zend_exception_get_default(TSRMLS_C);
		SPL_CE(InvalidArgumentException) = zend_exception_get_default(TSRMLS_C);
#	endif
#endif
	{
		zend_class_entry ce;
		INIT_CLASS_ENTRY(ce, "SennaException", NULL);
		SENNA_CE(SennaException) = zend_register_internal_class_ex(&ce, SPL_CE(RuntimeException), NULL TSRMLS_CC);
	}
	
	REGISTER_SENNA_CLASS(Senna, php_senna_functions, php_senna_new);
	
	REGISTER_SENNA_LONG_CONSTANT_ENCODINGS(Senna);
	REGISTER_SENNA_LONG_CONSTANT(Senna, "SUCCESS",              sen_success);
	REGISTER_SENNA_LONG_CONSTANT(Senna, "MEMORY_EXHAUSTED",     sen_memory_exhausted);
	REGISTER_SENNA_LONG_CONSTANT(Senna, "INVALID_FORMAT",       sen_invalid_format);
	REGISTER_SENNA_LONG_CONSTANT(Senna, "FILE_OPERATION_ERROR", sen_file_operation_error);
	REGISTER_SENNA_LONG_CONSTANT(Senna, "INVALID_ARGUMENT",     sen_invalid_argument);
	REGISTER_SENNA_LONG_CONSTANT(Senna, "OTHER_ERROR",          sen_other_error);
	/* ȉ̒萔Senna-1.0.9ł͒`ĂȂ */
	REGISTER_SENNA_LONG_CONSTANT(Senna, "EXTERNAL_ERROR",       sen_other_error + 1);
	REGISTER_SENNA_LONG_CONSTANT(Senna, "INTERNAL_ERROR",       sen_other_error + 2);
	REGISTER_SENNA_LONG_CONSTANT(Senna, "ABNORMAL_ERROR",       sen_other_error + 3);
	
	PHP_MINIT(senna_index)(INIT_FUNC_ARGS_PASSTHRU);
	PHP_MINIT(senna_records)(INIT_FUNC_ARGS_PASSTHRU);
	PHP_MINIT(senna_snippet)(INIT_FUNC_ARGS_PASSTHRU);
	
	if (sen_init() == sen_success) {
		return SUCCESS;
	} else {
		return FAILURE;
	}
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(senna) {
	/* uncomment this line if you have INI entries
	UNREGISTER_INI_ENTRIES();
	*/
	
	if (sen_fin() == sen_success) {
		return SUCCESS;
	} else {
		return FAILURE;
	}
}
/* }}} */

/* Remove if there's nothing to do at request start */
/* {{{ PHP_RINIT_FUNCTION
 */
PHP_RINIT_FUNCTION(senna)
{
	/* Apache̓쒆SennaꊷĂΉł悤ɂ邽߁ARINITœo^ */
	sen_info(
		&SENNA_G(api_version),
		NULL,
		NULL,
		&SENNA_G(default_encoding),
		NULL,
		NULL
	);
	REGISTER_SENNA_STRING_CONSTANT(Senna, "API_VERSION", SENNA_G(api_version));
	REGISTER_SENNA_LONG_CONSTANT(Senna, "DEFAULT_ENCODING", SENNA_G(default_encoding));
	
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request end */
/* {{{ PHP_RSHUTDOWN_FUNCTION
 */
PHP_RSHUTDOWN_FUNCTION(senna)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(senna)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "Senna support", "enabled");
	php_info_print_table_row(2, "Senna API version", SENNA_G(api_version));
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}
/* }}} */

/* The previous line is meant for vim and emacs, so it can correctly fold and 
   unfold functions in source code. See the corresponding marks just before 
   function definition, where the functions purpose is also documented. Please 
   follow this convention for the convenience of others editing your code.
*/


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
