
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "zend_exceptions.h"

#include <senna.h>
#include "php_senna.h"
#include "php_senna_in.h"

#include "senna_records.h"
#include "senna_index.h"

zend_object_handlers SENNA_HANDLER(Senna_Index);
PHPAPI zend_class_entry  *SENNA_CE(Senna_Index);

/* {{{ senna_index_free() free for Senna_Index object */
void senna_index_free(void *object TSRMLS_DC) { /* {{{ */
	senna_index_object *senna_index = (senna_index_object *) object;
	
	if (senna_index->index) {
		sen_index_close(senna_index->index);
	}
	
	if (senna_index->path) {
		efree(senna_index->path);
	}
	
	zend_object_std_dtor(&senna_index->std TSRMLS_CC);
	/* zend_hash_destroy(&intern->storage); */
	
	efree(object);
} /* }}} */

/* {{{ senna_index_new() allocate for Senna_Index object */
static zend_object_value senna_index_new(zend_class_entry *class_type TSRMLS_DC) {
	zend_object_value value;
	senna_index_object *senna_index;
	zval *tmp;
	
	senna_index = ecalloc(1, sizeof(*senna_index));
	senna_index->encoding = sen_enc_default;
	
	value.handlers = &SENNA_HANDLER(Senna_Index);
	value.handle = zend_objects_store_put(senna_index, (zend_objects_store_dtor_t) zend_objects_destroy_object, (zend_objects_free_object_storage_t) senna_index_free, NULL TSRMLS_CC);
	
	zend_object_std_init(&senna_index->std, class_type TSRMLS_CC);
	zend_hash_copy(senna_index->std.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));

	return value;
} /* }}} */

/* ޤZendƥ󥷥Ȥɬפˤʤؿ */
/* إѡؿ */

static int senna_index_get_create_args(INTERNAL_FUNCTION_PARAMETERS,
					const char **out_path, int *out_key_size, int *out_flags, int *out_initial_n_segments, sen_encoding *out_encoding) {
	const char *path;
	int path_len;
	long key_size = 0; /* 0: nullüβĹʸ */
	long flags = (long) SEN_INDEX_NORMALIZE;
	long initial_n_segments = 0;  /* 0: SennaΥǥե(SENNA_DEFAULT_INITIAL_N_SEGMENTS)ˤʤ */
	long encoding = (long) sen_enc_default;

	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
				"s|llll", &path, &path_len, &key_size, &flags, &initial_n_segments, &encoding)) {
		php_std_error_handling();
		return FAILURE;
	}

	php_std_error_handling();

	if (!path_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
				"Argument #1(path) cannot be empty");
		return FAILURE;
	}
	
	if ((key_size < 0) || (SEN_SYM_MAX_KEY_SIZE < key_size)) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
				"Argument #2(key size) must be 0 or 1..%d (0 means key size is variable length), %d given", SEN_SYM_MAX_KEY_SIZE, key_size);
		return FAILURE;
	}
	
	/* flagsä˥åʤ */
	
	if (initial_n_segments < 0) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
				"Argument #4(initial segments) must be greater than or equal to 0, %d given", initial_n_segments);
		return FAILURE;
	}
	
	switch ((sen_encoding) encoding) {
		case sen_enc_default:
		case sen_enc_none:
		case sen_enc_euc_jp:
		case sen_enc_utf8:
		case sen_enc_sjis:
		case sen_enc_latin1:
		case sen_enc_koi8r:
			/* no error */
			break;
		default:
			zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
				"Argument #5(encoding) must be one of "
				"Senna::ENC_DEFAULT, Senna::ENC_UTF8, Senna::ENC_EUC_JP, Senna::ENC_SJIS, "
				"Senna::ENC_LATIN1, Senna::ENC_KOI8R or Senna::ENC_NONE"
			);
			return FAILURE;
			break;
	}

	*out_path = path;
	*out_key_size = (int) key_size;
	*out_flags = (int) flags;
	*out_initial_n_segments = (int) initial_n_segments;
	*out_encoding = (sen_encoding) encoding;
	return SUCCESS;
}

static void senna_index_set_info(senna_index_object *target) {
	int pathsize;
	
	/* Ĺå뤿˰٥ХåեʤǸƤ */
	pathsize = sen_index_path(target->index, NULL, 0);
	
	/* ɤsen_index_path()ͤNULLޤ᤿byte֤褦(senna-1.1.1ǳǧ)strlen()ǤϤʤΤ-1 */
	target->path = (char *) ecalloc(pathsize, sizeof(*target->path));
	target->path_len = sen_index_path(target->index, target->path, pathsize) - 1;  
	/*php_printf("pathsize = %d, path_len = %d, path = %s\n", pathsize, strlen(path), path);*/

	sen_index_info(
		target->index,
		&target->key_size,
		&target->flags,
		&target->initial_n_segments,
		&target->encoding,
		NULL, NULL, NULL, NULL, NULL, NULL
	);
}

/* ᥽å */

/* {{{ proto void Senna_Index::__construct(string path[, int key_size = 0 [, int flags = Senna_Index::NORMALIZE [, int initial_n_segments = 0 [, int encoding = Senna::ENC_DEFAULT]]])
 *     Constructs a new Senna_Index from the index file path and the other args.
 *     If the index file does NOT exist, create a new index file.
 */
PHP_METHOD(Senna_Index, __construct) {
	const char *path;
	int key_size, flags, initial_n_segments;
	sen_encoding encoding;

	sen_index *index;
	senna_index_object *this;

	if (FAILURE == senna_index_get_create_args(INTERNAL_FUNCTION_PARAM_PASSTHRU,
						&path, &key_size, &flags, &initial_n_segments, &encoding)) {
		return;
	}
	
	/*
	 * pathsen_index_open()Ƥߤơ
	 *   sen_index != NULLʤ¸ΥǥåȤ
	 *   sen_index == NULLʤ鿷
	 * ǤʤзٹɽNULL֤
	 */
	index = sen_index_open(path);
	if (NULL == index) {
		index = sen_index_create(path, key_size, flags, initial_n_segments, encoding);
	} else {
		/* open()Ǥ硢¸index򸫤ƺΥץĴ٤ */
		sen_index_info(index, &key_size, &flags, &initial_n_segments, &encoding, NULL, NULL, NULL, NULL, NULL, NULL);
	}
	
	if (NULL == index) {
		/* ѡߥåcreateǤʤä */
		zend_throw_exception_ex(SENNA_CE(SennaException), sen_file_operation_error TSRMLS_CC, "failed to open/create an index file(%s): Permission denied or no such directory", path);
		return;
	}
	
	this = (senna_index_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	this->index = index;
	senna_index_set_info(this);
} /* }}} */

/* {{{ proto Senna_Index Senna_Index::create(string path[, int key_size = 0 [, int flags = Senna_Index::NORMALIZE [, int initial_n_segments = 0 [, int encoding = Senna_Index::ENC_DEFAULT]]])
 *     Create a new Senna_Index instance and a new index file from the path and the other args.
 */
PHP_METHOD(Senna_Index, create) {
	const char *path;
	int key_size, flags, initial_n_segments;
	sen_encoding encoding;

	sen_index *index;
	senna_index_object *senna_index;
	
	if (FAILURE == senna_index_get_create_args(INTERNAL_FUNCTION_PARAM_PASSTHRU,
						&path, &key_size, &flags, &initial_n_segments, &encoding)) {
		return;
	}

	index = sen_index_create(path, key_size, flags, initial_n_segments, encoding);
	if (NULL == index) {
		zend_throw_exception_ex(SENNA_CE(SennaException), sen_file_operation_error TSRMLS_CC, "failed to create an index file(%s): Permission denied or no such directory", path);
		return;
	}
	
	object_init_ex(return_value, SENNA_CE(Senna_Index));
	
	senna_index = (senna_index_object *) zend_object_store_get_object(return_value TSRMLS_CC);
	senna_index->index = index;
	senna_index_set_info(senna_index);
} /* }}} */

/* {{{ proto Senna_Index Senna_Index::open(string path) */
PHP_METHOD(Senna_Index, open) {
	char *path;
	int path_len;

	sen_index *index;
	senna_index_object *senna_index;
	
	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &path, &path_len)) {
		php_std_error_handling();
		return;
	}
	php_std_error_handling();

	if (!path_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(path) cannot be empty");
		return;
	}

	index = sen_index_open(path);
	if (NULL == index) {
		zend_throw_exception_ex(SENNA_CE(SennaException), sen_file_operation_error TSRMLS_CC, "failed to open an index file(%s): Permission denied or no such file or directory", path);
		return;
	}
	
	object_init_ex(return_value, SENNA_CE(Senna_Index));
	
	senna_index = (senna_index_object *) zend_object_store_get_object(return_value TSRMLS_CC);
	senna_index->index = index;
	senna_index_set_info(senna_index);
} /* }}} */

/* {{{ proto void Senna_Index::remove(string path) */
PHP_METHOD(Senna_Index, remove) {
	char *path;
	int path_len;
	
	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &path, &path_len)) {
		php_std_error_handling();
		return;
	}
	php_std_error_handling();

	if (!path_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(path) cannot be empty");
		return;
	}
	
	/*
	 * []
	 * Senna1.0.9(⡩)sen_index_remove()ϡ˽äƤ
	 * sen_file_operation_error֤Х롣
	 *
	 * ǥåκVGRAMץդʤȡ****.SEN.vե
	 * ʤremove()ؿϤäȤ롣
	 * Υե뤬¸ߤʤä硢ԤȤߤʤ
	 * sen_file_operation_error֤äƤͤˤʤäƤ
	 *
	 * ưrmȤä.SENեͤ¿ƽ줺˻ĤäƤġ
	 *
	 * [URL]
	 * http://lists.sourceforge.jp/mailman/archives/senna-dev/2007-September/000693.html
	 */
	{
		sen_rc result;
		
		result = sen_index_remove(path);
		if (php_version_compare(SENNA_G(api_version), "1.1.0") == -1) {
			/* 1.1.0ޤǤVGRAMե˴ؤХΤǡͤԤȽǤǤʤ */
			/* Ȥߤʤ */
			return;
		}
		
		if (sen_success != result) {
			zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_index_remove() returned error code %d", result);
			return;
		}
	}
} /* }}} */

/* {{{ proto Senna_Index::rename(string old_name, string new_name) */
PHP_METHOD(Senna_Index, rename) {
	char *old_name, *new_name;
	int old_name_len, new_name_len;
	
	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);
	
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
				"ss", &old_name, &old_name_len, &new_name, &new_name_len)) {
		php_std_error_handling();
		return;
	}

	php_std_error_handling();

	if (!old_name_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(old name) cannot be empty");
		return;
	}

	if (!new_name_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #2(new name) cannot be empty");
		return;
	}
	
	{
		sen_rc result = sen_index_rename(old_name, new_name);
		if (sen_success != result) {
			zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_index_rename() returned error code %d", result);
			return;
		}
	}
} /* }}} */

#ifdef PHP_SENNA_0
/* Senna_Index::query()Сsel()select()פʤΤǺʤ */
PHP_METHOD(Senna_Index, sel) {
	char *string;
	int string_len;
	
	senna_index_object *senna_index;
	sen_records *records;
	char keybuf[SEN_SYM_MAX_KEY_SIZE];
	int key_len;
	int score;

	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);
	
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &string, &string_len)) {
		php_std_error_handling();
		return;
	}

	php_std_error_handling();

	if (!string_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(query word) cannot be empty");
		return;
	}
	
	/* ƥ */
	senna_index = (senna_index_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	records = sen_index_sel(senna_index->index, string, string_len);
	while (sen_records_next(records, NULL, SEN_SYM_MAX_KEY_SIZE, NULL)) {
		key_len = sen_records_curr_key(records, keybuf, SEN_SYM_MAX_KEY_SIZE);
		score = sen_records_curr_score(records);
		php_printf("records['%s'].score = %d\n", keybuf, score);
	}
	
} /* }}} */
#endif

/* {{{ proto Senna_Records Senna_Index::query(string query [, int max_exprs = Senna_Index::DEFAULT_MAX_N_EXPRS])
 *     Return a Sena_Records object
 */
PHP_METHOD(Senna_Index, query) {
	const char *query_string;
	unsigned int query_string_len;
	int max_exprs;

	senna_index_object *senna_index = (senna_index_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	sen_encoding encoding = senna_index->encoding;
	
	senna_records_object *senna_records;

	if (FAILURE == senna_records_parse_parameters(INTERNAL_FUNCTION_PARAM_PASSTHRU,
				&query_string, &query_string_len, &max_exprs, &encoding TSRMLS_CC)) {
		return;
	}

#ifdef PHP_SENNA_0
	/* ϤäѤꤤʤ⡣顼ȯʤʤ뤫դˤ狼ˤ */
	if (!return_value_used) {
		/* ͤѤʤʤ顢ϼ¹ԤʤRecords֥Ȥ⥵ܤ */
		return;
	}
#endif
	
	object_init_ex(return_value, SENNA_CE(Senna_Records));
	
	senna_records = senna_records_constructor(
		return_value,
		getThis(),
		sen_rec_document,
		sen_rec_none,
		0 TSRMLS_DC
	);
	
	senna_records_exec_query(
		senna_records,
		query_string,
		query_string_len,
		sen_sel_or,
		max_exprs,
		encoding TSRMLS_DC
	);
}

/* {{{ proto void Senna_Index::upd(string key, string old_value, string new_value) */
PHP_METHOD(Senna_Index, upd) {
	char *key;
	char *old_value = NULL;
	char *new_value = NULL;
	int key_len, old_value_len, new_value_len;
	
	if (3 != ZEND_NUM_ARGS()) {
		/* ο㤦update()᥽åɤȴְäƤǽ롩 */
		zend_throw_exception_ex(SPL_CE(InvalidArgumentException), 0 TSRMLS_CC, "Senna_Index::upd() expects exactly 3 parameters, %d given", ZEND_NUM_ARGS());
		return;
	}

	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
				"sss", &key, &key_len, &old_value, &old_value_len, &new_value, &new_value_len)) {
		php_std_error_handling();
		return;
	}

	php_std_error_handling();

	if (!key_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(key name) cannot be empty");
		return;
	}
	
	/*
	ξȤȤפͤΥå򤷤ʤɤ
	if ((!old_value_len) && (!new_value_len)) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Either argument #2(old value) or #3(new value) must not be empty");
		return;
	}
	*/
	if ((!old_value_len) && (!new_value_len)) {
		/*
		 * ⤻᥽åɤȴ롣
		 * memo:
		 * ǥåkey¸ߤƤʤ
		 * sen_sym_del()sen_rc:4֤
		 * oldnewξȤˤʤϡޤkeyƤʤλ¿롣
		 * ξä˥᥽åɤȴƤޤäɤ
		 */
		RETURN_ZVAL(getThis(), 1, 0);
	}

	
	{
		senna_index_object *this = (senna_index_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	
		sen_rc result = sen_index_upd(
			this->index,
			key,
			(old_value_len) ? old_value : NULL, old_value_len,
			(new_value_len) ? new_value : NULL, new_value_len
		);
		
		if (sen_success != result) {
			zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_index_upd() returned error code %d", result);
			return;
		}
	}
	
	RETURN_ZVAL(getThis(), 1, 0);
} /* }}} */


/* {{{ proto void Senna_Index::update(string key, int section, string old_value, string new_value) */
PHP_METHOD(Senna_Index, update) {
	char *key, *old_string, *new_string;
	int key_len, old_string_len, new_string_len;
	int section;
	
	if (4 != ZEND_NUM_ARGS()) {
		/* ο㤦upd()᥽åɤȴְäƤǽ롩 */
		/*WRONG_PARAM_COUNT;*/
		zend_throw_exception_ex(SPL_CE(InvalidArgumentException), 0 TSRMLS_CC,
				"Senna_Index::update() expects exactly 4 parameters, %d given", ZEND_NUM_ARGS());
		return;
	}

	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
				"slss", &key, &key_len, &section, &old_string, &old_string_len, &new_string, &new_string_len)) {
		php_std_error_handling();
		return;
	}

	php_std_error_handling();

	if (!key_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(key name) cannot be empty");
		return;
	}
	
	if (section < 1) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #2(section number) must be greater than or equal to 1");
		return;
	}

	/*
	ξȤȤפͤΥå򤷤ʤɤ
	if ((!old_string_len) && (!new_string_len)) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Either argument #4(old value) or #5(new value) must not be empty");
		return;
	}
	*/
	if ((!old_string_len) && (!new_string_len)) {
		/* upd()Ʊ͡⤻᥽åɤȴ */
		RETURN_ZVAL(getThis(), 1, 0);
	}

	{
		sen_values *old_values = NULL;
		sen_values *new_values = NULL;
		senna_index_object *this;
		sen_rc result;
		
		/*
		 * upd()char*update()sen_valuesǻꤹɬפ餷Τ롣
		 * sectionνŤ߻ϡǼ˻ꤹ⸡˻ꤷĴ䤹ȻפΤ0ǸꤷƤ
		 * Ť߻򤷤ϡȯԻꤹ롣
		 */
		if (old_string_len) {
			old_values = sen_values_open();
	    	if ((result = sen_values_add(old_values, old_string, old_string_len, 0))) {
				zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_values_add() returned error code %d", result);
				return;
	    	}
	    }
	    if (new_string_len) {
			new_values = sen_values_open();
	    	if ((result = sen_values_add(new_values, new_string, strlen(new_string), 0))) {
				zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_values_add() returned error code %d", result);
				return;
	    	}
		}
		
		this = (senna_index_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((result = sen_index_update(this->index, key, section, old_values, new_values))) {
			zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_index_update() returned error code %d", result);
			return;
		}
	}
	
	RETURN_ZVAL(getThis(), 1, 0);
} /* }}} */

PHP_METHOD(Senna_Index, getPath) {
	senna_index_object *this = (senna_index_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_STRINGL(this->path, this->path_len, 1);
}
PHP_METHOD(Senna_Index, getKeySize) {
	senna_index_object *this = (senna_index_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) this->key_size);
}
PHP_METHOD(Senna_Index, getFlags) {
	senna_index_object *this = (senna_index_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) this->flags);
}
PHP_METHOD(Senna_Index, getInitialSegments) {
	senna_index_object *this = (senna_index_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) this->initial_n_segments);
}
PHP_METHOD(Senna_Index, getEncoding) {
	senna_index_object *this = (senna_index_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) this->encoding);
}

/* {{{ senna_index_functions[] */
zend_function_entry senna_index_functions[] = {
	PHP_ME(Senna_Index, __construct, NULL, ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
	
	PHP_ME(Senna_Index, create, NULL, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
	PHP_ME(Senna_Index, open,   NULL, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
	PHP_ME(Senna_Index, remove, NULL, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
	PHP_ME(Senna_Index, rename, NULL, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
	
	/*
	2Ĥquery()פȻפΤǺʤ⤷ɬס
	PHP_ME(Senna_Index, sel, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Index, select, NULL, ZEND_ACC_PUBLIC)
	*/
	PHP_ME(Senna_Index, query,  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Index, upd,    NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Index, update, NULL, ZEND_ACC_PUBLIC)
	
	PHP_ME(Senna_Index, getPath, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Index, getKeySize, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Index, getFlags, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Index, getInitialSegments, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Index, getEncoding, NULL, ZEND_ACC_PUBLIC)
	
	/*
	 ꥢ饤
	PHP_ME(PDO, __wakeup,		NULL,					ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
	PHP_ME(PDO, __sleep,		NULL,					ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
	*/
	{NULL, NULL, NULL}
};
/* }}} */


PHP_MINIT_FUNCTION(senna_index) {
	unsigned int initial_n_segments;
	sen_info(NULL, NULL, NULL, NULL, &initial_n_segments, NULL);
	
	REGISTER_SENNA_CLASS(Senna_Index, senna_index_functions, senna_index_new);

	/* Senna_Index::NORMALIZEʤɡPDO::FETCH_COLUMNΤ褦ʽ񼰤ǻȤ */
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "NORMALIZE", SEN_INDEX_NORMALIZE);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "SPLIT_ALPHA", SEN_INDEX_SPLIT_ALPHA);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "SPLIT_DIGIT", SEN_INDEX_SPLIT_DIGIT);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "SPLIT_SYMBOL", SEN_INDEX_SPLIT_SYMBOL);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "MORPH_ANALYSE", SEN_INDEX_MORPH_ANALYSE);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "NGRAM", SEN_INDEX_NGRAM);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "DELIMITED", SEN_INDEX_DELIMITED);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "ENABLE_SUFFIX_SEARCH", SEN_INDEX_ENABLE_SUFFIX_SEARCH);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "DISABLE_SUFFIX_SEARCH", SEN_INDEX_DISABLE_SUFFIX_SEARCH);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "WITH_VGRAM", SEN_INDEX_WITH_VGRAM);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "WITH_VACUUM", SEN_INDEX_WITH_VACUUM);
	
	/* key_size0ꤹȡʸIDϲĹʸˤʤ */
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "VARIABLE_KEY_SIZE", 0);
	/* key_sizeˤϾ¤ */
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "MAX_KEY_SIZE", SEN_SYM_MAX_KEY_SIZE);
	/* initial_n_segmentsȤؿ0ꤹȡǥեȤ512(DEFAULT_INITIAL_N_SEGMENTS) * 256KB = 128MBΥХåեݤ */
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "DEFAULT_INITIAL_N_SEGMENTS", initial_n_segments);

	/*
	 * ȯԻκ黻ҿ(ñ)ϡǥեȤ32Ȥ롣
	 * Ludia, Tritonn, Google32äΤǤ򻲹ͤˤ
	 * Senna_Records::DEFAULT_MAX_N_EXPRSȽʣƤΤǽ񤭴ݤsenna_records.c
	 */
	REGISTER_SENNA_LONG_CONSTANT(Senna_Index, "DEFAULT_MAX_N_EXPRS", PHP_SENNA_DEFAULT_MAX_N_EXPRS);

	return SUCCESS;
}


/* The previous line is meant for vim and emacs, so it can correctly fold and 
   unfold functions in source code. See the corresponding marks just before 
   function definition, where the functions purpose is also documented. Please 
   follow this convention for the convenience of others editing your code.
*/


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
