
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "zend_exceptions.h"
#include "zend_interfaces.h"

#include <senna.h>
#include "php_senna.h"
#include "php_senna_in.h"

#include "senna_index.h"
#include "senna_records_row.h"
#include "senna_records.h"

/* SPLͭʤ顢Countable󥿡եȤ */
#ifdef PHP_SENNA_CAN_DEPENDS_ON_SPL
extern PHPAPI zend_class_entry *SPL_CE(Countable);
#endif

zend_object_handlers SENNA_HANDLER(Senna_Records);
PHPAPI zend_class_entry  *SENNA_CE(Senna_Records);

/* If you declare any globals in senna_result.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(senna_result)
*/

/* {{{ senna_records_free() free for Senna_Records object */
void senna_records_free(void *object TSRMLS_DC) {
	senna_records_object *senna_records = (senna_records_object *) object;
	
	if (senna_records->records) {
		sen_records_close(senna_records->records);
	}
	
	if (senna_records->zval_senna_index) {
		zval_ptr_dtor(&senna_records->zval_senna_index);
	}
	zend_object_std_dtor(&senna_records->std TSRMLS_CC);

	efree(object);
} /* }}} */

/* {{{ senna_records_new() allocate for Senna_Records object */
static zend_object_value senna_records_new(zend_class_entry *class_type TSRMLS_DC) {
	zend_object_value value;
	senna_records_object *senna_records;
	zval *tmp;
	
	senna_records = ecalloc(1, sizeof(*senna_records));

	value.handlers = &SENNA_HANDLER(Senna_Records);
	value.handle = zend_objects_store_put(senna_records, (zend_objects_store_dtor_t) zend_objects_destroy_object, (zend_objects_free_object_storage_t) senna_records_free, NULL TSRMLS_CC);
	
	zend_object_std_init(&senna_records->std, class_type TSRMLS_CC);
	zend_hash_copy(senna_records->std.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));

	return value;
} /* }}} */


/* foreachIteratorؿ */
static int senna_records_rewind_and_sort(senna_records_object *this);

typedef struct _senna_records_iterator {
	zend_object_iterator it;
	senna_records_object *senna_records;
	ulong index;
	const sen_recordh *rh;
} senna_records_iterator;

static void senna_records_iterator_dtor(zend_object_iterator *it TSRMLS_DC) {
	senna_records_iterator *I = (senna_records_iterator *) it;
	I->senna_records->in_iteration = 0;
	if (I->it.data) {
		zval_ptr_dtor((zval **) &I->it.data);
	}
	efree(I);
}

static int senna_records_iterator_valid(zend_object_iterator *it TSRMLS_DC) {
	senna_records_iterator *I = (senna_records_iterator *) it;
	return  (I->rh) ? SUCCESS : FAILURE;
}

static void senna_records_iterator_current_data(zend_object_iterator *it, zval ***data TSRMLS_DC) {
	senna_records_iterator *I = (senna_records_iterator *) it;
	zval *zval_senna_records_row;
	
	if (!I->rh) {
		*data = NULL;
		return;
	}
	
	MAKE_STD_ZVAL(zval_senna_records_row);
	object_init_ex(zval_senna_records_row, SENNA_CE(Senna_Records_Row));
	senna_records_row_constructor(zval_senna_records_row, (zval *) I->it.data TSRMLS_CC);
	
	*data = &zval_senna_records_row;
}

static int senna_records_iterator_current_key(zend_object_iterator *it, char **str_key, uint *str_key_len, ulong *int_key TSRMLS_DC) {
	senna_records_iterator *I = (senna_records_iterator *) it;

	if (!I->rh) {
		return HASH_KEY_NON_EXISTANT;
	}
	
	*int_key = I->index;
	return HASH_KEY_IS_LONG;
}

static void senna_records_iterator_move_forwards(zend_object_iterator *it TSRMLS_DC) {
	senna_records_iterator *I = (senna_records_iterator *) it;

	I->index++;

	if (sen_records_next(I->senna_records->records, NULL, 0, NULL)) {
		I->rh = sen_records_curr_rec(I->senna_records->records);
	} else {
		I->rh = NULL;
	}
}

static void senna_records_iterator_rewind(zend_object_iterator *it TSRMLS_DC) {
	senna_records_iterator *I = (senna_records_iterator *) it;

#if PHP_SENNA_0
	int nhits;
	sen_rc result;


	sen_records_rewind(I->senna_records->records);

	if (I->senna_records->sorted) {
		I->senna_records->records->curr_rec = NULL;
		/*
		senna-1.1.1Ǥϡsort쥳ɤrewindǤʤ
		if ((result = sen_records_rewind(I->senna_records->records))) {
			zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_records_rewind() returned error code %d", result);
			break;
		}
		*/
	} else {
		if (nhits = sen_records_nhits(I->senna_records->records)) {
			/* ߽˥Ȥ */
			if ((result = sen_records_sort(I->senna_records->records, (I->senna_records->limit) ? I->senna_records->limit : nhits, NULL))) {
				zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_records_sort() returned error code %d", result);
				return;
			}
			I->senna_records->sorted = 1;
		}
	}
#endif
	senna_records_rewind_and_sort(I->senna_records);

	I->index = 0;
	if (sen_records_next(I->senna_records->records, NULL, 0, NULL)) {
		I->rh = sen_records_curr_rec(I->senna_records->records);
	} else {
		I->rh = NULL;
	}
}


static zend_object_iterator_funcs senna_records_iterator_funcs = {
	senna_records_iterator_dtor,
	senna_records_iterator_valid,
	senna_records_iterator_current_data,
	senna_records_iterator_current_key,
	senna_records_iterator_move_forwards,
	senna_records_iterator_rewind
};

zend_object_iterator *senna_records_get_iterator(zend_class_entry *ce, zval *object, int by_ref TSRMLS_DC) {
	senna_records_iterator *I;
	
	if (by_ref) {
		zend_error(E_ERROR, "An iterator cannot be used with foreach by reference");
	}

	I = ecalloc(1, sizeof(*I));
	I->senna_records = (senna_records_object *) zend_object_store_get_object(object TSRMLS_CC);
	I->senna_records->in_iteration = 1;
	
	I->index = 0;
	I->it.funcs = &senna_records_iterator_funcs;
	I->it.data = (void *) object;
	zval_add_ref(&object);
	
	return (zend_object_iterator *) I;
}
/* }}} */

/* ޤZendƥ󥷥Ȥɬפˤʤؿ */
/* إѡؿ */

senna_records_object *senna_records_constructor(zval *target, zval *zval_senna_index,
			sen_rec_unit record_unit, sen_rec_unit sub_record_unit, int max_n_subrecs TSRMLS_DC) {
	switch (record_unit) {
		case sen_rec_document:
		case sen_rec_section:
		case sen_rec_position:
		/* case sen_rec_none:̥쥳ɤNONEꤷᡪ */
			/* no error */
			break;
		default:
			zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #2(record unit) must be one of Senna_Records::DOCUMENT, Senna_Records::SECTION or Senna_Records::POSITION");
			return NULL;
			break;
	}
	
	switch (sub_record_unit) {
		/* case sen_rec_document:֥쥳ɤDOCUMENTꤷᡪ */
		case sen_rec_section:
		case sen_rec_position:
			/* no error */
			break;
		case sen_rec_none:
			max_n_subrecs = 0;
			/* no error */
			break;
		default:
			zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #3(sub record unit) must be one of Senna_Records::SECTION, Senna_Records::POSITION or Senna_Records::NONE");
			return NULL;
			break;
	}

	if (sub_record_unit <= record_unit) {
		/* ֥쥳ñ̤ϡ쥳ñ̤⾮ʤФʤʤ */
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #3(sub record unit) must be smaller unit than Argument #2(record unit)");
		return NULL;
	}

	if (max_n_subrecs < 0) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #4(max subrecords) must be greater than or equal to 0");
		return NULL;
	}

	{
		senna_records_object *senna_records = (senna_records_object *) zend_object_store_get_object(target TSRMLS_CC);

		senna_records->records = sen_records_open(record_unit, sub_record_unit, (unsigned int) max_n_subrecs);
		if (NULL == senna_records->records) {
			/* NULL֤Τϥ­λΤߡ */
			zend_throw_exception_ex(SENNA_CE(SennaException), sen_memory_exhausted TSRMLS_CC, "sen_records_open() returned NULL");
			return NULL;
		}
		
		senna_records->max_n_subrecords = (unsigned int) max_n_subrecs;
		senna_records->record_unit = record_unit;
		senna_records->sub_record_unit = sub_record_unit;
		
		/* Records󥹥󥹤Index󥹥󥹤򻲾Ȥ롣
		 * ƥ󥷥¦Υɤѿ򻲾Ȥϡ
		 * ưrefcount+1ʤФʤʤ餷
		 */
		senna_records->zval_senna_index = zval_senna_index;
		zval_add_ref(&zval_senna_index);
		
		return senna_records;
	}
}

/* query¹ѥ᥽åɶ̤ΰåؿ */
int senna_records_parse_parameters(INTERNAL_FUNCTION_PARAMETERS,
			const char **query_string, unsigned int *query_string_len, int *max_exprs, sen_encoding *encoding TSRMLS_DC) {
	*max_exprs = PHP_SENNA_DEFAULT_MAX_N_EXPRS;

	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
						"s|ll", (char **) query_string, (int *) query_string_len, (long *) max_exprs, (long *) encoding /* ͽ */)) {
		php_std_error_handling();
		return FAILURE;
	}
	
	php_std_error_handling();

	if (!*query_string_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(query string) cannot be empty");
		return FAILURE;
	}

	if (*max_exprs < 1) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #2(max expressions) must be greater than or equal to 1");
		return FAILURE;
	}
	
	switch (*encoding) {
		case sen_enc_default:
		case sen_enc_none:
		case sen_enc_euc_jp:
		case sen_enc_utf8:
		case sen_enc_sjis:
		case sen_enc_latin1:
		case sen_enc_koi8r:
			/* no error */
			break;
		default:
			zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
				"Argument #3(query encoding) must be one of "
				"Senna::ENC_DEFAULT, Senna::ENC_UTF8, Senna::ENC_EUC_JP, Senna::ENC_SJIS, "
				"Senna::ENC_LATIN1, Senna::ENC_KOI8R or Senna::ENC_NONE"
			);
			return FAILURE;
			break;
	}
	
	return SUCCESS;
}

int senna_records_exec_query(senna_records_object *this,
			const char *query_string, unsigned int query_string_len,
					sen_sel_operator sel_operator, int max_exprs, sen_encoding encoding TSRMLS_DC) {
	sen_query *query;
	
	if (this->in_iteration) {
		/*
		 * ƥ졼ιϵĤʤ
		 * (JavaˤConcurrentModificationExceptionȯ㳰ȯǤϤʤfatal error)
		 *
		 * [memo]
		 * ̥쥳ɤϡsen_records_next()Ƭ1鷺ļͤˤʤäƤ롣
		 * Τᡢforeach()롼Ǥȿrecords֥ȤȤѤȡ
		 * 뤫狼ʤեեȤΤˡǥ顼ȯ롣
		 * senna-1.1.1Ǥϡsen_query_exec()Ƥ֤ȥ쥳ɼѤ뤬Ƭ뤿ᡢ
		 * ̵¥롼פˤʤä
		 * Х򸡽Ф뤿ᡢå顼ȯ롣
		 */
		zend_throw_exception_ex(SPL_CE(BadMethodCallException), 0 TSRMLS_CC, "Cannot execute query in a foreach loop");
		/*php_error_docref(NULL TSRMLS_CC, E_ERROR, "You should not execute query in a foreach loop");*/
		return FAILURE;
	}
	
	/*
	 * [memo]
	 * ΥХǥ󥰤Ǥϡsen_sel_andǥեȤα黻Ҥꤷ
	 * ̾θ󥸥Ʊưˤ롣
	 * LudiaTritonnsen_sel_orǥեȤˤʤäƤΤǡա
	 * 
	 * [ΥХǥ󥰤ǡsen_sel_orǥեȤˤʤäͳ]
	 * ɤȤandȻפΤǥǥեȤandˤorϾʤΤǤϡ
	 *   ޤȤʤorǥեȤȡץ¦ǥƬɬץ饰ޤ򵭽Ҥ뤫
	 *   ñƬ+䴰ʤФʤʤʤꡢȤƤȤˤ
	 * 
	 * [Sennaϡsen_sel_orǥեȤˤʤäƤͳ]
	 * ͽ:
	 * 1. Υ⸡ϳ줬ФȤˤʤΤ⤷ʤ
	 * 2. ȤꤢORơٹʤȤΤ侩ƤΤ⤷ʤ
	 *    recordsФƹʤ߸ǤͤˤʤäƤ롣
	 * 3. ְ̤ǤORȤƤΤ⤷ʤ
	 * 4. ORɤȤʬǳȯƤΤ⤷ʤŪͳ
	 * 5. ñ˥ƥȡ˷᤿Τ⤷ʤߴѹǤʤ
	 *
	 * [OR򤷤ϤɤȤ]
	 * Ȥꤢ󸡺̤
	 * ޤϰANDƤߤơʤȻפä饭ɤĴ뤫
	 *   ORǺƸ٤ǽ餫ORΤϡǿͤˤϥǤʤ餷
	 * ŬΨƤܤʤ
	 * ɤ¿СΥޤޤƤƤ줽
	 * ƸΨܤ
	 * õʸ񸡺㤬褯Ф褦ˡ˴ؤƲǽʸ¤Ĵ
	 *   ORŻ뤵Τ⤷ʤ
	 *
	 * 󥸥Τ褦˥ʸ򰷤ʤ¤ꡢ
	 *   OR򤹤ŬΨʤ갭ʤΤǤϡ
	 * Ƹ줬ƴϢΤñȤǤСOR򤷤Ƥ
	 *   ñ줬Ʊʸ˴ޤޤΨ⤯ANDȤʤѤʤ
	 *   ̤뵤롣
	 * Ĥͳ䤤碌Ƥߤ
	 */
	query = sen_query_open(query_string, query_string_len, sen_sel_and /* 嵭ͳˤand */, max_exprs, encoding);
	this->ignored_query_string_len = sen_query_rest(query, (const char ** const) &this->ignored_query_string);
	this->sorted = 0;  /* ¹Ԥ뤿ӡ̤Ⱦ֤ */

	/*
	 * ⡧֥֥쥳ɡפꤹȤɤʤΤ
	 * ꤷפ˱ơ쥳ɤ˥°롣
	 *
	 * 㤨С
	 * main: sen_rec_document
	 * sub:  sen_rec_section
	 * ξ硢ƥᥤ쥳ɤϥޥåʸˤʤꡢ
	 * ƥ֥쥳ɤϤɤsectionǥޥåǼ롣
	 *
	 * [query¹Ի]
	 * 󥳡ɤλȥǥåΥ󥳡ɤʤȡ˻ꤷʸˤäƤ
	 * ̤տޤʤΤˤʤ롣꤯Ǥʸ⤢ġ
	 */
	
	/* ʲ¹Խ */
	{
		senna_index_object *senna_index = (senna_index_object *) zend_object_store_get_object(this->zval_senna_index TSRMLS_CC);
		/*
		 * Senna-1.1.1ߡsen_query_exec()Ǹ̤äƤ쥳ɤɲáǤΤsen_rec_documentꤷΤ
		 * URL:[Senna-dev 799] http://lists.sourceforge.jp/mailman/archives/senna-dev/2008-March/000798.html
		 * ʲΥɤquery.c:sen_query_exec()ؿΥɤȰʣƤ롣
		 * Υ쥳ɤ˸̤졢¸θ̤ȥޡ륢르ꥺˤʤäƤ롣
		 */
		int n = sen_records_nhits(this->records);
		sen_records *s = (n) ? sen_records_open(this->record_unit, this->sub_record_unit, this->max_n_subrecords) : this->records;
		sen_rc result;

		if ((result = sen_query_exec(senna_index->index, query, s, sen_sel_or))) {
			zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_query_exec() returned error code %d", result);
			return FAILURE;
		}
		
		if (n) {
			/* sen_records_union()sen_records_close(s);Ƥ */
			switch (sel_operator) {
				case sen_sel_or:
					if (!sen_records_union(this->records, s)) { sen_records_close(s); }
					break;
				case sen_sel_and:
					if (!sen_records_intersect(this->records, s)) { sen_records_close(s); }
					break;
				case sen_sel_but:
					if (!sen_records_subtract(this->records, s)) { sen_records_close(s); }
					break;
				case sen_sel_adjust:
					/* Senna-1.1.1ߡޤƤʤ */
					break;
				default:
					sen_records_close(s);
					break;
			}
		}
	}
	sen_query_close(query);
	
	return SUCCESS;
}

/* add(), adjust(), remove(), match()Ƥ */
static int senna_records_parse_parameters_and_exec_query(INTERNAL_FUNCTION_PARAMETERS, sen_sel_operator sel_operator TSRMLS_DC) {
	const char *query_string;
	unsigned int query_string_len;
	int max_exprs;
	sen_encoding encoding;
	
	senna_records_object *this = (senna_records_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	senna_index_object *senna_index = (senna_index_object *) zend_object_store_get_object(this->zval_senna_index TSRMLS_CC);
	encoding = senna_index->encoding;

	if (FAILURE == senna_records_parse_parameters(INTERNAL_FUNCTION_PARAM_PASSTHRU,
						&query_string, &query_string_len, &max_exprs, &encoding TSRMLS_CC)) {
		return FAILURE;
	}
	
	return senna_records_exec_query(
		this,
		query_string,
		query_string_len,
		sel_operator,
		max_exprs,
		encoding TSRMLS_DC
	);
}

static int senna_records_get_count(senna_records_object *this) {
	int nhits = sen_records_nhits(this->records);
	
	/* limitꤵƤơºݤμ쥳ɿ꾮limit֤ͤ */
	if (this->limit && (this->limit < nhits)) {
		return this->limit;
	} else {
		return nhits;
	}
}

/* ߽˥Ȥ */
static int senna_records_rewind_and_sort(senna_records_object *this) {
	int nhits;
	sen_rc result;

	sen_records_rewind(this->records);

	if (this->sorted) {
		/*
		senna-1.1.1Ǥϡsort쥳ɤrewindǤʤ
		if ((result = sen_records_rewind(this->records))) {
			zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_records_rewind() returned error code %d", result);
			return FAILURE;
		}
		*/
		this->records->curr_rec = NULL;
	} else {
		if (nhits = sen_records_nhits(this->records)) {
			/* ߽˥Ȥ */
			if ((result = sen_records_sort(this->records, (this->limit) ? this->limit : nhits, NULL))) {
				zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_records_sort() returned error code %d", result);
				return FAILURE;
			}
		}
		this->sorted = 1;
	}

	return SUCCESS;
}

/* ᥽å */

/* {{{ proto void Senna_Records::__construct(Senna_Index index [, int record_unit = Senna_Records::DOCUMENT [, int sub_records_unit = Senna_Records::NONE [, int max_n_subrecs = Senna_Records::DEFAULT_MAX_N_SUBRECS]]])
 *     Constructs a new Senna_Records from a Senna_Index instance and the other args.
 * @params record_unit ̤Υ쥳ɤȤߤñ̡
 *     Senna_Records::DOCUMENTꤹȡ̤ʸñ̤ˤʤ롣
 *     Senna_Records::SECTIONꤹȡ̤ϤʸΤsectionñ̤ˤʤ롣
 *     Senna_Records::POSITIONꤹȡ̤ϤʸΤsectionΤи֤Ȥñ̤ˤʤ롣
 *     и֤Ȥϡmecabʬ񤭤ݤβ֥åܤ///ɤ/ŷ//ע:pos=0, ŷ:pos=3
 * @params sub_records_unit ̤γƥ쥳ɤˡɲþȤƻñ̡
 *     Senna_Records::SECTIONꤹȡʸβܤsectionǥޥå狼褦ˤʤ롣
 *     Senna_Records::POSITIONꤹȡʸβܤsectionβܤνи֤ǥޥå狼褦ˤʤ롣
 *     Senna_Records::NONEꤹȡɲþϻʤ
 * @params max_n_subrecs ƥ쥳ɤĥ֥쥳ɤκ
 *     sub_records_unitSenna_Records::NONEꤷ硢ΰ̵뤹롣
 * 
 */
PHP_METHOD(Senna_Records, __construct) {
	zval *zval_senna_index;
	sen_rec_unit record_unit = sen_rec_document;
	sen_rec_unit sub_record_unit = sen_rec_none;
	long max_n_subrecs = PHP_SENNA_RECORDS_DEFAULT_MAX_N_SUBRECS;

	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
		"O|lll",
		&zval_senna_index, SENNA_CE(Senna_Index),
		(long *) &record_unit,
		(long *) &sub_record_unit,
		&max_n_subrecs)
	) { 
		php_std_error_handling();
		return;
	}

	php_std_error_handling();
	
	senna_records_constructor(
		getThis(),
		zval_senna_index,
		record_unit,
		sub_record_unit,
		(int) max_n_subrecs TSRMLS_CC
	);
}
/* }}} */

/* {{{ proto int Senna_Records::getIgnoredQueryString()
   Returns the query string which is ignored for the reason why the length of the query string is too long. */
PHP_METHOD(Senna_Records, getIgnoredQueryString) {
	senna_records_object *senna_records = (senna_records_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	
	RETURN_STRINGL(senna_records->ignored_query_string, senna_records->ignored_query_string_len, 1);
}
/* }}} */

/* {{{ proto int Senna_Records::scoreOf(string key)
   Returns the score value of the record which corresponds to given key in the records.
   If key does not exist in the records, returns 0.
   [] ƥ졼Ѥư뤿ᡢforeach롼פǸƤ٤ʤ(Ƥ֤BadMethodCallExceptionꤲ)
 */
PHP_METHOD(Senna_Records, scoreOf) {
	char *key;
	int key_len;
	
	senna_records_object *senna_records;
	
	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &key, &key_len)) {
		php_std_error_handling();
		return;
	}

	php_std_error_handling();
	
	senna_records = (senna_records_object *) zend_object_store_get_object(getThis() TSRMLS_CC);

	if (senna_records->in_iteration) {
		/*
		 * ƥ졼scoreOf()᥽åɤϵĤʤ
		 * SennaԶڡ(http://qwik.jp/senna/knownproblems.html)ˤȡ
		 * sen_records_find, sen_records_atˤĤ
		 * sen_records_find, sen_records_at¹Ԥʹߤˡ sen_records_next¹Ԥ
		 * ̤֤뤳Ȥݾڤޤ sen_records_rewind¹Ԥƥɬפޤ
		 * ȤΤȤʤΤǡХȯʤ褦ƥ졼Υ϶ػߤ롣
		 */
		zend_throw_exception_ex(SPL_CE(BadMethodCallException), 0 TSRMLS_CC, "This method cannot be called in a foreach loop");
		/*php_error_docref(NULL TSRMLS_CC, E_ERROR, "You should not call this method in a foreach loop");*/
		return;
	}

	if (!key_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(key name) cannot be empty");
		return;
	}
	
	RETURN_LONG(sen_records_find(senna_records->records, key));
}
/* }}} */

/* {{{ proto int Senna_Records::slice(int offset, int count)
   offsetܤΥǥåcountĤθ̥쥳ɤ֤
   γǤSenna_Records_Row֥ȤˤʤäƤ롣
   ϰϳoffsetꤷƤ㳰ꤲʤñ˶֤(ȤפΤ)
   [] ƥ졼Ѥư뤿ᡢforeach롼פǸƤ٤ʤ(Ƥ֤BadMethodCallExceptionꤲ)
 */
PHP_METHOD(Senna_Records, slice) {
	long offset;
	long count;
	
	senna_records_object *this;
	
	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &offset, &count)) {
		php_std_error_handling();
		return;
	}

	php_std_error_handling();
	
	this = (senna_records_object *) zend_object_store_get_object(getThis() TSRMLS_CC);

	if (this->in_iteration) {
		/*
		 * ƥ졼scoreOf()᥽åɤϵĤʤ
		 * SennaԶڡ(http://qwik.jp/senna/knownproblems.html)ˤȡ
		 * sen_records_find, sen_records_atˤĤ
		 * sen_records_find, sen_records_at¹Ԥʹߤˡ sen_records_next¹Ԥ
		 * ̤֤뤳Ȥݾڤޤ sen_records_rewind¹Ԥƥɬפޤ
		 * ȤΤȤʤΤǡХȯʤ褦ƥ졼Υ϶ػߤ롣
		 */
		zend_throw_exception_ex(SPL_CE(BadMethodCallException), 0 TSRMLS_CC, "This method cannot be called in a foreach loop");
		return;
	}

	if (offset < 0) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(offset) must be greater than or equal to 0");
		return;
	}
	
	if (count < 1) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #2(count) must be greater than or equal to 1");
		return;
	}
	
	array_init(return_value);
	{
		zval *zval_row;
		int i;
		int max = senna_records_get_count(this);
		int limit = (offset + count < max) ? offset + count : max;
		
		/*sen_records_rewind(this->records);*/
		senna_records_rewind_and_sort(this);
		
		for (i = 0; i < offset; i++) {
			/* offsetܤޤskip */
			sen_records_next(this->records, NULL, 0, NULL);
		}
		for (i = offset; i < limit; i++) {
			/* offsetܤcountĤΥ쥳ɤ */
			MAKE_STD_ZVAL(zval_row);
			object_init_ex(zval_row, SENNA_CE(Senna_Records_Row));

			sen_records_next(this->records, NULL, 0, NULL);
			senna_records_row_constructor(zval_row, getThis() TSRMLS_CC);

			add_next_index_zval(return_value, zval_row);
		}
	}
}
/* }}} */

/* {{{ proto void Senna_Records::setLimit(int count)
   ̿ξ¤countĤꤹ
 */
PHP_METHOD(Senna_Records, setLimit) {
	long limit;
	
	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &limit)) {
		php_std_error_handling();
		return;
	}

	php_std_error_handling();
	
	if (limit < 0) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(limit count) must be greather than or equal to 0 (0 means unlimited), %d given", limit);
		return;
	}
	
	{
		senna_records_object *this = (senna_records_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
		if (this->limit != (int) limit) {
			this->limit = (int) limit;
			this->sorted = 0;
		}
	}
}
/* }}} */

/* {{{ proto void Senna_Records::getLimit()
   ̿ξ¤롣̵¤ξ0֤
 */
PHP_METHOD(Senna_Records, getLimit) {
	senna_records_object *this = (senna_records_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) this->limit);
}
/* }}} */

/* {{{ proto int Senna_Records::count()
   Returns the number of records in a buffered record set. */
PHP_METHOD(Senna_Records, count) {
	senna_records_object *this = (senna_records_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) senna_records_get_count(this));
}
/* }}} */

/* {{{ proto Senna_Records Senna_Records::add(string query [, int max_exprs = Senna_Records::DEFAULT_MAX_N_EXPRS])
   󥹥󥹤ݻƤ쥳ɤˡqueryˤ븡̥쥳ɤɲä(½)
   ʬȤ֤$records->add('hoge')->add('abcd');Τ褦fluent interface뤿ᡣ
   [] ƥ졼Ѥư뤿ᡢforeach롼פǸƤ٤ʤ(Ƥ֤BadMethodCallExceptionꤲ)
 */
PHP_METHOD(Senna_Records, add) {
	if (FAILURE == senna_records_parse_parameters_and_exec_query(INTERNAL_FUNCTION_PARAM_PASSTHRU, sen_sel_or TSRMLS_CC)) {
		return;
	}
	/* Fluent interfaceΤ褦˻ȤˤϡʬȤ֤餷 */
	RETURN_ZVAL(getThis(), 1, 0);
} /* }}} */

/* {{{ proto Senna_Records Senna_Records::adjust(string query [, int max_exprs = Senna_Records::DEFAULT_MAX_N_EXPRS])
   󥹥󥹤ݻƤ쥳ɤˡqueryˤ븡̤ޤޤƤФΥ쥳ɤΥͤû롣
   ƼʬȤ֤
   [] ƥ졼Ѥư뤿ᡢforeach롼פǸƤ٤ʤ(Ƥ֤BadMethodCallExceptionꤲ)
 */
PHP_METHOD(Senna_Records, adjust) {
	if (FAILURE == senna_records_parse_parameters_and_exec_query(INTERNAL_FUNCTION_PARAM_PASSTHRU, sen_sel_adjust TSRMLS_CC)) {
		return;
	}
	RETURN_ZVAL(getThis(), 1, 0);
} /* }}} */

/* {{{ proto Senna_Records Senna_Records::match(string query [, int max_exprs = Senna_Records::DEFAULT_MAX_N_EXPRS])
   󥹥󥹤ݻƤ쥳ɤФơʤ߸򤹤(̽)
   queryˤ븡̥쥳ɤȥޥåΤĤޥåʤäΤϺ롣
   ƼʬȤ֤
   [] ƥ졼Ѥư뤿ᡢforeach롼פǸƤ٤ʤ(Ƥ֤BadMethodCallExceptionꤲ)
 */
PHP_METHOD(Senna_Records, match) {
	if (FAILURE == senna_records_parse_parameters_and_exec_query(INTERNAL_FUNCTION_PARAM_PASSTHRU, sen_sel_and TSRMLS_CC)) {
		return;
	}

	/* Fluent interfaceΤ褦˻ȤˤϡʬȤ֤餷 */
	RETURN_ZVAL(getThis(), 1, 0);
} /* }}} */

/* {{{ proto Senna_Records Senna_Records::remove(string query [, int max_exprs = Senna_Records::DEFAULT_MAX_N_EXPRS])
   󥹥󥹤ݻƤ쥳ɤФơ򤹤()
   queryˤ븡̥쥳ɤȥޥåΤ(ե륿)ޥåʤäΤĤ
   ƼʬȤ֤
   [] ƥ졼Ѥư뤿ᡢforeach롼פǸƤ٤ʤ(Ƥ֤BadMethodCallExceptionꤲ)
 */
PHP_METHOD(Senna_Records, remove) {
	if (FAILURE == senna_records_parse_parameters_and_exec_query(INTERNAL_FUNCTION_PARAM_PASSTHRU, sen_sel_but TSRMLS_CC)) {
		return;
	}

	/* Fluent interfaceΤ褦˻ȤˤϡʬȤ֤餷 */
	RETURN_ZVAL(getThis(), 1, 0);
} /* }}} */

zend_function_entry senna_records_functions[] = {
	PHP_ME(Senna_Records, __construct, NULL, ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
	PHP_ME(Senna_Records, getIgnoredQueryString, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records, scoreOf, NULL, ZEND_ACC_PUBLIC)
	
	PHP_ME(Senna_Records, slice, NULL, ZEND_ACC_PUBLIC)
	
	PHP_ME(Senna_Records, setLimit,  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records, getLimit,  NULL, ZEND_ACC_PUBLIC)
	
	PHP_ME(Senna_Records, add,    NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records, adjust, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records, match,  NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records, remove, NULL, ZEND_ACC_PUBLIC)
	
	/*
	PHP_ME(Senna_Records, union, NULL, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
	...
	*/
	
	/* implements Countable */
	PHP_ME(Senna_Records, count, NULL, ZEND_ACC_PUBLIC)
	
	/* implements Iterator */
	/* ͤ */
	
	{NULL, NULL, NULL}
};

PHP_MINIT_FUNCTION(senna_records) {
	REGISTER_SENNA_CLASS(Senna_Records, senna_records_functions, senna_records_new);
	SENNA_CE(Senna_Records)->get_iterator = senna_records_get_iterator;

#ifdef PHP_SENNA_CAN_DEPENDS_ON_SPL
	zend_class_implements(SENNA_CE(Senna_Records) TSRMLS_CC, 2, zend_ce_traversable, SPL_CE(Countable));
#else
	zend_class_implements(SENNA_CE(Senna_Records) TSRMLS_CC, 1, zend_ce_traversable);
#endif

	REGISTER_SENNA_LONG_CONSTANT(Senna_Records, "DOCUMENT", sen_rec_document);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Records, "SECTION", sen_rec_section);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Records, "POSITION", sen_rec_position);
	/* ̤ REGISTER_SENNA_LONG_CONSTANT(Senna_Records, "USERDEF", sen_rec_userdef); */
	REGISTER_SENNA_LONG_CONSTANT(Senna_Records, "NONE", sen_rec_none);

	REGISTER_SENNA_LONG_CONSTANT(Senna_Records, "DEFAULT_MAX_N_SUBRECS", PHP_SENNA_RECORDS_DEFAULT_MAX_N_SUBRECS);
	
	/* Senna_Index::DEFAULT_MAX_N_EXPRSƱ͡ʣƤΤǽ񤭴ݤsenna_index.c */
	REGISTER_SENNA_LONG_CONSTANT(Senna_Records, "DEFAULT_MAX_N_EXPRS", PHP_SENNA_DEFAULT_MAX_N_EXPRS);
	
	
	/* ˥ͥȸƤӽФʲΥ饹ϡPHP¦ǥ󥹥󥹤Ǥʤ */
	PHP_MINIT(senna_records_row)(INIT_FUNC_ARGS_PASSTHRU);
	return SUCCESS;
}


/* The previous line is meant for vim and emacs, so it can correctly fold and 
   unfold functions in source code. See the corresponding marks just before 
   function definition, where the functions purpose is also documented. Please 
   follow this convention for the convenience of others editing your code.
*/


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
