
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "zend_exceptions.h"
#include "zend_interfaces.h"

#include <senna.h>
#include "php_senna.h"
#include "php_senna_in.h"

#include "senna_records.h"
#include "senna_records_subrecords.h"
#include "senna_records_row.h"

#define PHP_SENNA_METHOD_CALL(classname, name) PHP_MN(classname##_##name)(INTERNAL_FUNCTION_PARAM_PASSTHRU)

zend_object_handlers SENNA_HANDLER(Senna_Records_Row);
PHPAPI zend_class_entry  *SENNA_CE(Senna_Records_Row);

/* If you declare any globals in senna_result.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(senna_result)
*/

/* {{{ senna_records_row_free() free for Senna_Records_Row object */
void senna_records_row_free(void *object TSRMLS_DC) {
	senna_records_row_object *senna_records_row = (senna_records_row_object *) object;
	
	if (senna_records_row->zval_senna_records) {
		zval_ptr_dtor(&senna_records_row->zval_senna_records);
	}
	if (senna_records_row->key) {
		efree(senna_records_row->key);
	}
	
	zend_object_std_dtor(&senna_records_row->std TSRMLS_CC);

	efree(object);
} /* }}} */

/* {{{ senna_records_row_new() allocate for Senna_Records_Row object */
static zend_object_value senna_records_row_new(zend_class_entry *class_type TSRMLS_DC) {
	zend_object_value value;
	senna_records_row_object *senna_records_row;
	zval *tmp;
	
	senna_records_row = ecalloc(1, sizeof(*senna_records_row));

	value.handlers = &SENNA_HANDLER(Senna_Records_Row);
	value.handle = zend_objects_store_put(senna_records_row, (zend_objects_store_dtor_t) zend_objects_destroy_object, (zend_objects_free_object_storage_t) senna_records_row_free, NULL TSRMLS_CC);
	
	zend_object_std_init(&senna_records_row->std, class_type TSRMLS_CC);
	zend_hash_copy(senna_records_row->std.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));

	return value;
} /* }}} */


/* ޤZendƥ󥷥Ȥɬפˤʤؿ */
/* إѡؿ */

/* {{{ senna_records_row_constructor() creates a new instance for Senna_Records_Row object
 * ƥ󥷥Senna_Records_Row󥹥ȥ饯
 * 1˻ꤷzval*ѿSenna_Records_Row֥ȤȤƽ롣
 * Senna_Records饹IteratorƤ֡ʳǥ֥Ȥ뤳ȤϤǤʤ
 */
senna_records_row_object *senna_records_row_constructor(zval *zval_senna_records_row, zval *zval_senna_records TSRMLS_DC) {
	senna_records_row_object *senna_records_row;
	senna_records_object *senna_records;
	
	char keybuf[SEN_SYM_MAX_KEY_SIZE];
	sen_rc result;
	
	senna_records_row = (senna_records_row_object *) zend_object_store_get_object(zval_senna_records_row TSRMLS_CC);
	senna_records = (senna_records_object *) zend_object_store_get_object(zval_senna_records TSRMLS_CC);
	
	senna_records_row->row = sen_records_curr_rec(senna_records->records);
	if (! senna_records_row->row) {
		/* NULL֤Τsenna_records->records == NULLλ(senna_records->records == NULLȤϤꤨʤ⡩) */
		zend_throw_exception_ex(SENNA_CE(SennaException), sen_invalid_argument TSRMLS_CC, "sen_records_curr_rec() returned NULL");
		return;
	}
	
	/* Records_Row󥹥󥹤Records󥹥󥹤򻲾Ȥ롣 */
	senna_records_row->zval_senna_records = zval_senna_records;
	zval_add_ref(&zval_senna_records);
	
	result = sen_record_info(
		senna_records->records,
		senna_records_row->row,
		keybuf,
		SEN_SYM_MAX_KEY_SIZE,
		&senna_records_row->keysize, /* strlen(key) + 1֤롣keyNULLüʸξϡNULL1byteޤ᤿Ĺ֤äƤ */
		&senna_records_row->section,
		&senna_records_row->position,
		&senna_records_row->score,
		(int *) &senna_records_row->n_subrecords
	);
	
	if (sen_success != result) {
		zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_record_info() returned error code %d", result);
		return NULL;
	}
	
	/* ⡧
	 * sen_record_info()ǼǤn_subrecsͤϡSenna_Records֥˻ꤷ
	 * max_n_subrecs礭ͤˤʤ뤳Ȥ⤢褦
	 * ºݤˤmax_n_subrecsǻꤷͤĶ륪եåȤ
	 * sen_record_subrec_info()Ǽ褦Ȥinvalid argumentˤʤ롣
	 * տޤưˤ뤿ᡢmax_n_subrecsĶͤʤmax_n_subrecs֤롣
	 */
	if (senna_records->max_n_subrecords < senna_records_row->n_subrecords) {
		senna_records_row->n_subrecords = senna_records->max_n_subrecords;
	}
	
	/*
	 * keyNULLüɤˤ餺NULLüʸˤʤ롣
	 * NULLüä硢1NULLɲä롣
	 */
	senna_records_row->key = estrndup(keybuf, senna_records_row->keysize);  /* ϥǥȥ饯 */
	/* php_printf("strlen(key) = %d, key_len = %d\n", strlen(senna_records_row->key), senna_records_row->key_len); */
	
	return senna_records_row;
}

/* Senna_Records_Row饹Υ᥽å */

/* {{{ proto private void Senna_Records_Row::__construct()
 * Senna_Records_Row֥ȤϡPHP¦ǤϼưǤʤ
 * Senna_Records饹ǤΤ롣
 */
PHP_METHOD(Senna_Records_Row, __construct) {
	zend_throw_exception_ex(SPL_CE(BadMethodCallException), 0 TSRMLS_CC, "You cannot create an instance of Senna_Records_Row manually");
	/*php_error_docref(NULL TSRMLS_CC, E_ERROR, "You should not create a Senna_Records_Row instance manually");*/
}
/* }}} */

/* {{{ proto string Senna_Records_Row::getKey() */
PHP_METHOD(Senna_Records_Row, getKey) {
	senna_records_row_object *this = (senna_records_row_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	
	if (this->key[this->keysize] == '\0') {
		RETURN_STRINGL(this->key, this->keysize - 1, 1);
	} else {
		RETURN_STRINGL(this->key, this->keysize, 1);
	}
} /* }}} */

/* {{{ proto int Senna_Records_Row::getKeySize() */
PHP_METHOD(Senna_Records_Row, getKeySize) {
	senna_records_row_object *this = (senna_records_row_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	
	/* keysizestrlen(key)ĹȤϰ㤤NULLޤǴޤbytekeyĹ򼨤Ƥ롣 */
	RETURN_LONG((long) this->keysize);
} /* }}} */

/* {{{ proto int Senna_Records_Row::getScore() */
PHP_METHOD(Senna_Records_Row, getScore) {
	senna_records_row_object *this = (senna_records_row_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) this->score);
} /* }}} */

/* {{{ proto int Senna_Records_Row::getSection() */
PHP_METHOD(Senna_Records_Row, getSection) {
	senna_records_row_object *this = (senna_records_row_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) this->section);
} /* }}} */

/* {{{ proto int Senna_Records_Row::getPosition() */
PHP_METHOD(Senna_Records_Row, getPosition) {
	senna_records_row_object *this = (senna_records_row_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) this->position);
} /* }}} */

/* {{{ proto Senna_Records_Subrecords Senna_Records_Row::getSubRecords() */
PHP_METHOD(Senna_Records_Row, getSubRecords) {
	object_init_ex(return_value, SENNA_CE(Senna_Records_SubRecords));
	senna_records_subrecords_constructor(return_value, getThis() TSRMLS_DC);
} /* }}} */

/* {{{ proto int Senna_Records_Row::getSubRecordCount() */
PHP_METHOD(Senna_Records_Row, getSubRecordCount) {
	senna_records_row_object *this = (senna_records_row_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) this->n_subrecords);
} /* }}} */

/* {{{ proto bool Senna_Records_Row::offsetGet(string hash_key)
 * Ex. $section = $row['section'];
 */
PHP_METHOD(Senna_Records_Row, offsetGet) {
	char *type;
	int type_len;
	
	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &type, &type_len)) {
		php_std_error_handling();
		return;
	}

	php_std_error_handling();

	{
		/* type = 'key', 'keysize', 'score', 'section', 'position', 'subrecords', 'subrecordscount' Τɤ줫ʸʸ̤ʤ */
		char *lc_type = ecalloc(type_len + 1, sizeof(*lc_type));
		zend_str_tolower_copy(lc_type, type, type_len);

		/* ɲáˤoffsetExists()ˤ */
		if (strcmp(lc_type, "key") == 0) {
			PHP_SENNA_METHOD_CALL(Senna_Records_Row, getKey);
		} else if (strcmp(lc_type, "keysize") == 0) {
			PHP_SENNA_METHOD_CALL(Senna_Records_Row, getKeySize);
		} else if (strcmp(lc_type, "score") == 0) {
			PHP_SENNA_METHOD_CALL(Senna_Records_Row, getScore);
		} else if (strcmp(lc_type, "section") == 0) {
			PHP_SENNA_METHOD_CALL(Senna_Records_Row, getSection);
		} else if (strcmp(lc_type, "position") == 0) {
			PHP_SENNA_METHOD_CALL(Senna_Records_Row, getPosition);
		} else if (strcmp(lc_type, "subrecords") == 0) {
			PHP_SENNA_METHOD_CALL(Senna_Records_Row, getSubRecords);
		} else if (strcmp(lc_type, "subrecordcount") == 0) {
			PHP_SENNA_METHOD_CALL(Senna_Records_Row, getSubRecordCount);
		} else {
			/*
			 * ǤΥϥåΥ̾δְ㤤ϡץޤץߥƤåߥβǽ⤤
			 * ΤᡢInvalidArgumentExceptionUnexpectedValueExceptionŬڤȻפ
			 * ᥽åɸƤӽФ㤨ȡ$row->getScore();$row->getScoer();Ƚ񤭴ְ㤨Ƥޤä˻Ƥ롣
			 */
			zend_throw_exception_ex(SPL_CE(InvalidArgumentException), 0 TSRMLS_CC, "Argument #1(hash key) must be one of \"key\", \"keysize\", \"score\", \"section\", \"position\", \"subrecords\" or \"subrecordcount\" (case insensitive), \"%s\" given", type);
		}
		
		efree(lc_type);
	}
}
/* }}} */

/* {{{ proto bool Senna_Records_Row::offsetExists(string type)
 * Ex. isset($row['keysize']); ȤϾʤȻפġ
 */
PHP_METHOD(Senna_Records_Row, offsetExists) {
	char *type;
	int type_len;
	
	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &type, &type_len)) {
		php_std_error_handling();
		return;
	}

	php_std_error_handling();

	{
		char *lc_type = ecalloc(type_len + 1, sizeof(*lc_type));
		zend_str_tolower_copy(lc_type, type, type_len);

		/* ɲáˤoffsetGet()ˤ */
		if (strcmp(lc_type, "key") == 0) {
			RETVAL_TRUE;
		} else if (strcmp(lc_type, "keysize") == 0) {
			RETVAL_TRUE;
		} else if (strcmp(lc_type, "score") == 0) {
			RETVAL_TRUE;
		} else if (strcmp(lc_type, "section") == 0) {
			RETVAL_TRUE;
		} else if (strcmp(lc_type, "position") == 0) {
			RETVAL_TRUE;
		} else if (strcmp(lc_type, "subrecords") == 0) {
			RETVAL_TRUE;
		} else if (strcmp(lc_type, "subrecordcount") == 0) {
			RETVAL_TRUE;
		} else {
			RETVAL_FALSE;
		}
		
		efree(lc_type);
	}
}
/* }}} */

PHP_METHOD(Senna_Records_Row, offsetSet) {
	/* Records_Row֥ȤѹԲĤʤΤǡsetter̤ */
	zend_throw_exception_ex(SPL_CE(BadMethodCallException), 0 TSRMLS_CC, "Senna_Records_Row properties are read only. This method is not implemented");
	/*php_error_docref(NULL TSRMLS_CC, E_ERROR, "Senna_Records_Row object is read only. Senna_Records_Row::offsetSet() is not implemented");*/
}

PHP_METHOD(Senna_Records_Row, offsetUnset) {
	/* Records_Row֥ȤѹԲĤʤΤǡoffsetUnset()᥽åɤ̤ */
	zend_throw_exception_ex(SPL_CE(BadMethodCallException), 0 TSRMLS_CC, "Senna_Records_Row properties are read only. This method is not implemented");
	/*php_error_docref(NULL TSRMLS_CC, E_ERROR, "Senna_Records_Row object is read only. Senna_Records_Row::offsetUnset() is not implemented");*/
}

/* implements ArrayAccessΤˡꤹ */
static ZEND_BEGIN_ARG_INFO_EX(arginfo_offsetGet, 0, 0, 1)
	ZEND_ARG_INFO(0, index)
ZEND_END_ARG_INFO()

static ZEND_BEGIN_ARG_INFO_EX(arginfo_offsetSet, 0, 0, 2)
	ZEND_ARG_INFO(0, index)
	ZEND_ARG_INFO(0, newval)
ZEND_END_ARG_INFO()

zend_function_entry senna_records_row_functions[] = {
	PHP_ME(Senna_Records_Row, __construct,  NULL, ZEND_ACC_PRIVATE | ZEND_ACC_CTOR)
	
	PHP_ME(Senna_Records_Row, getKey,          NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records_Row, getKeySize,      NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records_Row, getScore,        NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records_Row, getSection,      NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records_Row, getPosition,     NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records_Row, getSubRecords,   NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records_Row, getSubRecordCount, NULL, ZEND_ACC_PUBLIC)
	
	/* implements ArrayAccess */
	PHP_ME(Senna_Records_Row, offsetGet,    arginfo_offsetGet, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records_Row, offsetSet,    arginfo_offsetSet, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records_Row, offsetUnset,  arginfo_offsetGet, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Records_Row, offsetExists, arginfo_offsetGet, ZEND_ACC_PUBLIC)
	{NULL, NULL, NULL}
};

PHP_MINIT_FUNCTION(senna_records_row) {
	REGISTER_SENNA_CLASS(Senna_Records_Row, senna_records_row_functions, senna_records_row_new);
	zend_class_implements(SENNA_CE(Senna_Records_Row) TSRMLS_CC, 1, zend_ce_arrayaccess);

	/* ʲΥ饹ϡPHP¦Υɤǥ󥹥󥹤Ǥʤ */
	PHP_MINIT(senna_records_subrecords)(INIT_FUNC_ARGS_PASSTHRU);

	return SUCCESS;
}


/* The previous line is meant for vim and emacs, so it can correctly fold and 
   unfold functions in source code. See the corresponding marks just before 
   function definition, where the functions purpose is also documented. Please 
   follow this convention for the convenience of others editing your code.
*/


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
