
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "zend_exceptions.h"

#include <senna.h>
#include "php_senna.h"
#include "php_senna_in.h"

#include "senna_snippet.h"

zend_object_handlers SENNA_HANDLER(Senna_Snippet);
PHPAPI zend_class_entry  *SENNA_CE(Senna_Snippet);

typedef struct {
	unsigned int width;
	int changed;
	char *query;
	int query_len;
} query_term_args_t;

/* {{{ senna_snippet_free() free for Senna_Snippet object */
void senna_snippet_free(void *object TSRMLS_DC) {
	senna_snippet_object *senna_snippet = (senna_snippet_object *) object;
	
	if (senna_snippet->snip) {
		sen_snip_close(senna_snippet->snip);
	}
	
	if (senna_snippet->tag_list) {
		/*
		ʸzval_ptr_dtor()ǼưǲΤå
		zval **str;
		if (SUCCESS == zend_hash_index_find(HASH_OF(senna_snippet->tag_list), 0, (void**)&str)) {
			php_printf("before: %s\n", Z_STRVAL_PP(str));
		}
		*/
		zval_ptr_dtor(&senna_snippet->tag_list);
		/*
		ΤʸѿƤ롣
		php_printf("after: %s\n", Z_STRVAL_PP(str));
		*/
	}
	
	/* createQuerySnippet()᥽åɤǥ󥹥󥹤ϡ˥ݻƤ */
	if (senna_snippet->open_tag_list) {
		if (senna_snippet->open_tag_list->str_list) {
			efree(senna_snippet->open_tag_list->str_list);
		}
		if (senna_snippet->open_tag_list->len_list) {
			efree(senna_snippet->open_tag_list->len_list);
		}
		efree(senna_snippet->open_tag_list);
	}
	if (senna_snippet->close_tag_list) {
		if (senna_snippet->close_tag_list->str_list) {
			efree(senna_snippet->close_tag_list->str_list);
		}
		if (senna_snippet->close_tag_list->len_list) {
			efree(senna_snippet->close_tag_list->len_list);
		}
		efree(senna_snippet->close_tag_list);
	}
	
	zend_object_std_dtor(&senna_snippet->std TSRMLS_CC);

	efree(object);
} /* }}} */

/* {{{ senna_snippet_new() allocate for Senna_snippet object */
static zend_object_value senna_snippet_new(zend_class_entry *class_type TSRMLS_DC) {
	zend_object_value value;
	senna_snippet_object *senna_snippet;
	zval *tmp;
	
	senna_snippet = ecalloc(1, sizeof(*senna_snippet));

	/*
	 * SennaλͤǤϡϥĤsen_snip_close()ޤ
	 * ƤӽФ¦ݻƤʤФʤʤΤݻ롣
	 */
	MAKE_STD_ZVAL(senna_snippet->tag_list);
	array_init(senna_snippet->tag_list);

	value.handlers = &SENNA_HANDLER(Senna_Snippet);
	value.handle = zend_objects_store_put(senna_snippet, (zend_objects_store_dtor_t) zend_objects_destroy_object, (zend_objects_free_object_storage_t) senna_snippet_free, NULL TSRMLS_CC);
	
	zend_object_std_init(&senna_snippet->std, class_type TSRMLS_CC);
	zend_hash_copy(senna_snippet->std.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));

	return value;
} /* }}} */

static int senna_snippet_validate_constructor_args(
			unsigned int i, sen_encoding encoding, long flags, long width, long max_results, zval *zval_mapping) {
	
	switch (encoding) {
		case sen_enc_default:
		case sen_enc_none:
		case sen_enc_euc_jp:
		case sen_enc_utf8:
		case sen_enc_sjis:
		case sen_enc_latin1:
		case sen_enc_koi8r:
			/* no error */
			break;
		default:
			zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
				"Argument #%d(encoding) must be one of "
				"Senna::ENC_DEFAULT, Senna::ENC_UTF8, Senna::ENC_EUC_JP, Senna::ENC_SJIS, "
				"Senna::ENC_LATIN1, Senna::ENC_KOI8R or Senna::ENC_NONE",
				i
			);
			return FAILURE;
	}
	
	/*
	 * senna-1.1.1ɲä줿SEN_SNIP_COPY_TAGϡǥեȤΥ򥳥ԡʤХΤǥե饰ΤλѤػߤ
	 * [Senna-dev 792] http://lists.sourceforge.jp/mailman/archives/senna-dev/2008-February/000791.html
	 */
	if (flags & SEN_SNIP_COPY_TAG) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
			"Argument #%d(flags) must be Senna_Snippet::NORMALIZE or NULL. "
			"In order to avoid the senna bug, you cannot use 0x0002(SEN_SNIP_COPY_TAG)",
			i + 1
		);
		return FAILURE;
	}
	
	if (width < 1) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
			"Argument #%d(width) must be greater than or equal to 1, %d given",
			i + 2,
			width
		);
		return FAILURE;
	}

	/* 1 <= max_results <= MAX_SNIP_RESULT_COUNTϰϳǤ4֤ */
	if ((max_results < 1) || (MAX_SNIP_RESULT_COUNT < max_results)) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
			"Argument #%d(max results) must be 1 <= max_results <= %d(Senna_Snippet::MAX_RESULT_COUNT), %d given",
			i + 3,
			MAX_SNIP_RESULT_COUNT,
			max_results
		);
		return FAILURE;
	}
	
	if (!zval_mapping || (IS_NULL == Z_TYPE_P(zval_mapping))) {
		/* ok */
	} else if (IS_LONG == Z_TYPE_P(zval_mapping)) {
		/* ok */
	} else {
		/* arrayϤ줿ѿηθʤΤUnexpectedValueExceptionǤϤʤInvalidArgumentExceptionꤲ */
		zend_throw_exception_ex(SPL_CE(InvalidArgumentException), 0 TSRMLS_CC,
			"Argument #%d(mapping) must be Senna_Snippet::MAPPING_NONE or Senna_Snippet::MAPPING_COMPAT",
			i + 6
		);
		return FAILURE;
	}
	
	return SUCCESS;
}

static
/* Senna_snippet饹Υ᥽å */

/* {{{ proto void Senna_Snippet::__construct(int encoding, int flags, int width [, int max_results = 1 [, string defaultOpenTag = '' [, string defaultCloseTag = '' [, mixed mapping = Senna_Snippet::MAPPING_NONE]]]])
   Constructs a new Senna_Snippet.
   []
   encoding: Senna::ENC_DEFAULT, Senna::ENC_NONE, Senna::ENC_EUC_JP, Senna::ENC_UTF8 or Senna::ENC_SJIS
   flags:    Senna_Snippet::NORMALIZE or NULL
   width:    SnippetΡɽΡ׺Byte(ĤĹ˴ޤʤ)(memo/Senna_Snippet.txt:1)ͤꤹ롣
   max_results:     snippetоʸ󤫤snippetκĿ1Senna_Snippet::MAX_RESULT_COUNTޤǤ͡ǥե:1
   defaultOpenTag:  snippetоʸθñˤĤʸ󡣥ǥե:ʸ
   defaultCloseTag: snippetоʸθñθˤĤʸ󡣥ǥե:ʸ
   mapping:  Senna_Snippet::MAPPING_NONE (HTMLƥƥѴʤǥե)
             or Senna_Snippet::MAPPING_COMPAT (&"<>4Ĥ򥨥󥳡ɤ롣줾&amp;&quot;&lt;&gt;ˤʤ)
 */
PHP_METHOD(Senna_Snippet, __construct) {
	sen_encoding encoding;
	long flags;
	long width;
	
	long max_results = 1;
	char *defaultOpenTag = NULL;
	int defaultOpenTag_len = 0;
	char *defaultCloseTag = NULL;
	int defaultCloseTag_len = 0;
	zval *zval_mapping = NULL;
	
	sen_snip_mapping *mapping;

	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);
	
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
		"lll|lssz",
		(long *) &encoding, &flags, &width,
		&max_results,
		&defaultOpenTag, &defaultOpenTag_len,
		&defaultCloseTag, &defaultCloseTag_len,
		&zval_mapping)
	) { 
		php_std_error_handling();
		return;
	}
	
	php_std_error_handling();
	
	if (FAILURE == senna_snippet_validate_constructor_args(1, encoding, flags, width, max_results, zval_mapping)) {
		return;
	}

	if (!zval_mapping || (IS_NULL == Z_TYPE_P(zval_mapping))) {
		mapping = (sen_snip_mapping *) NULL;
	} else {
		mapping = (sen_snip_mapping *) Z_LVAL_P(zval_mapping);
	}

	{
		/* ʲ2Ĥʸϡzval_ptr_dtor(&this->tag_list)(Zendefree()ƤǤ) */
		char *dup_defaultOpenTag = (defaultOpenTag_len) ? estrndup(defaultOpenTag, defaultOpenTag_len) : NULL;
		char *dup_defaultCloseTag = (defaultCloseTag_len) ? estrndup(defaultCloseTag, defaultCloseTag_len) : NULL;
		
		senna_snippet_object *this = (senna_snippet_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
		
		/* ѿݻŪʤΤǡduplicateƤϤʤʤ0ꤹ */
		add_next_index_stringl(this->tag_list, dup_defaultOpenTag, defaultOpenTag_len, 0);
		add_next_index_stringl(this->tag_list, dup_defaultCloseTag, defaultCloseTag_len, 0);
		
		this->snip = sen_snip_open(
			encoding,
			(int) flags,
			(unsigned int) width,
			(unsigned int) max_results,
			dup_defaultOpenTag, (unsigned int) defaultOpenTag_len,
			dup_defaultCloseTag, (unsigned int) defaultCloseTag_len,
			mapping
		);
		
		if (NULL == this->snip) {
			/* NULL֤Τϡ­λʤΤsen_memory_exhausted򥻥åȤ(max_resultsϥåѤߤΤʤϤ) */
			zend_throw_exception_ex(SENNA_CE(SennaException), sen_memory_exhausted TSRMLS_CC, "sen_snip_open() returned NULL");
			return;
		}
		
		/* ĹʾΥɤɲäǤʤ */
		this->width = width;
	}
}
/* }}} */

/* {{{ proto Senna_Snippet Senna_Snippet::addKeyword(string keyword [, string openTag = '' [, string closeTag = '']])
 *     Snippetɤɲä롣
 *     ʬȤ֤
 *     $snip->addKeyword('keyword1')->addKeyword('keyword2');Τ褦˻Ȥ
 */
PHP_METHOD(Senna_Snippet, addKeyword) {
	char *keyword;
	int keyword_len;
	
	char *openTag = NULL;
	int openTag_len = 0;
	char *closeTag = NULL;
	int closeTag_len = 0;
	
	senna_snippet_object *this;

	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);
	/*
	 * zend_parse_parametersΥɥ
	 * http://jp.php.net/manual/ja/internals2.ze1.zendapi.php
	 * ˤϡ!׻ϡaoOr  z ˤĤƤΤŬѲǽפȽ񤤤Ƥ뤬
	 * PHP5.0.0ʹߤzend_API.c:399դ򸫤ƤߤȡstringǤȤ롣
	 */
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
		"s|s!s!", &keyword, &keyword_len, &openTag, &openTag_len, &closeTag, &closeTag_len)
	) { 
		php_std_error_handling();
		return;
	}
	php_std_error_handling();
	
	this = (senna_snippet_object *) zend_object_store_get_object(getThis() TSRMLS_CC);

	if (MAX_SNIP_COND_COUNT <= this->keyword_count) {
		/* ɲäǤ륭ɿ(MAX_SNIP_COND_COUNT)ĤãƤ顢Sennaλ;⤦ɲäǤʤ */
		zend_throw_exception_ex(SPL_CE(BadMethodCallException), 0 TSRMLS_CC, "The number of keywords must be less than or equal to %d(Senna_Snippet::MAX_KEYWORD_COUNT)", MAX_SNIP_COND_COUNT);
		return;
	}

	if (!keyword_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(keyword) cannot be empty");
		return;
	}
	
	if (this->width < keyword_len) {
		/* 󥹥ȥ饯ꤷwidthХȤĶĹΥɤɲäǤʤ */
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
				"Argument #1(keyword) length exceeds the maximum allowed length of %d bytes", this->width);
		return;
	}
	
	{
		sen_rc result;
		char *dup_keyword = estrndup(keyword, keyword_len);
		char *dup_openTag = (openTag) ? estrndup(openTag, openTag_len) : NULL;
		char *dup_closeTag = (closeTag) ? estrndup(closeTag, closeTag_len) : NULL;
		
		add_next_index_stringl(this->tag_list, dup_keyword, keyword_len, 0);
		add_next_index_stringl(this->tag_list, dup_openTag, openTag_len, 0);
		add_next_index_stringl(this->tag_list, dup_closeTag, closeTag_len, 0);
		
		result = sen_snip_add_cond(
			this->snip,
			dup_keyword, keyword_len,
			dup_openTag, openTag_len, 
			dup_closeTag, closeTag_len
		);
		
		if (sen_success == result) {
			this->keyword_count++;
		} else {
			zend_throw_exception_ex(SENNA_CE(SennaException), result TSRMLS_CC, "sen_snip_add_cond() returned error code %d", result);
			return;
		}
		
	}
	RETURN_ZVAL(getThis(), 1, 0);
}
/* }}} */

/* {{{ proto int Senna_Snippet::getKeywordCount()
       addKeyword()Ͽɿ֤
       
       createQuerySnippet()ǥ󥹥󥹤ϡΥɤ
       ŪaddKeyword()뤬ߤνꤳΥɿϼ뤳ȤϤǤʤ
       Sennaλͤˤꡢsnip->cond_lenɤˡʤ
       ΤᡢcreateQuerySnippet()󥹥󥹤Ͻ֤
       createQuerySnippet()2(max_exprs)ǻꤷ֤ͤ
 */
PHP_METHOD(Senna_Snippet, getKeywordCount) {
	senna_snippet_object *this = (senna_snippet_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
	RETURN_LONG((long) this->keyword_count);
	/*RETURN_LONG((long) this->snip->cond_len);sen_snip¤Τsnip.hƤΤǳcond_lenɤʤä*/
}


/* {{{ proto array Senna_Snippet::execute(string target)
   targetͿʸ󤫤snippet(KWIC)snippetȤ֤
   ֤ǿϡ(󥹥ȥ饯ǻꤷ)max_resultsĤȤʤ롣
   Returns array of snippets.
   []
   SennaΥХͤ狼ʤʲΤ褦ˤʤ롣
   󥹥ȥ饯flagsSenna_Snippet::NORMALIZEꤹȡ
   this is senna.פsnippetϡaddKeyword('senna', '<b>', '</b>');¹ԤƤ
   this is<b> senna</b>.פȤʤ롣
   NORMALIZEꤷʤ
   this is <b>senna</b>.פȤʤ롣
   ɤ˶򤬴ޤޤƤȡζ˳դ
 */
PHP_METHOD(Senna_Snippet, execute) {
	const char *target;
	unsigned int target_len;

	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", (char *) &target, (long *) &target_len)) { 
		php_std_error_handling();
		return;
	}
	php_std_error_handling();
	
	if (!target_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(target string) cannot be empty");
		return;
	}
	
	{
		int i;
		char *snippet;
		unsigned int snippet_len = 0;
		
		unsigned int n_results = 0;
		unsigned int max_tagged_len = 0;
		senna_snippet_object *this = (senna_snippet_object *) zend_object_store_get_object(getThis() TSRMLS_CC);

		sen_rc rc = sen_snip_exec(this->snip, target, target_len, &n_results, &max_tagged_len);
		
		if (sen_success != rc) {
			zend_throw_exception_ex(SENNA_CE(SennaException), rc TSRMLS_CC, "sen_snip_exec() returned error code %d", rc);
			return;
		}
		
		array_init(return_value);
		snippet = (char *) ecalloc(max_tagged_len, sizeof(*snippet));
		for (i = 0; i < n_results; i++) {
			rc = sen_snip_get_result(this->snip, i, snippet, &snippet_len);
			if (sen_success == rc) {
				add_next_index_stringl(return_value, snippet, snippet_len, 1);
			} else {
				/* Senna-1.1.1ߡԤ븶sen_invalid_argumentΤߤʤΤǤˤʤϤ */
				zend_throw_exception_ex(SENNA_CE(SennaException), rc TSRMLS_CC, "sen_snip_get_result() returned error code %d", rc);
				return;
			}
		}
		efree(snippet);
	}
}
/* }}} */

/* ʲcreateQuerySnippet()᥽åѥإѡؿ */

static php_senna_snippet_tag_list_t *senna_snippet_create_tag_list(zval *zval_list, zval *bank) {
	php_senna_snippet_tag_list_t *tag_list;
	zval **item;
	HashPosition hp;
	int i;
	
	tag_list = (php_senna_snippet_tag_list_t *) ecalloc(1, sizeof(*tag_list));
	tag_list->count = zend_hash_num_elements(Z_ARRVAL_P(zval_list));
	
	tag_list->str_list = (char **) ecalloc(tag_list->count, sizeof(tag_list->str_list));
	tag_list->len_list = (unsigned int *) ecalloc(tag_list->count, sizeof(*tag_list->len_list));

	for (
		i = 0, zend_hash_internal_pointer_reset_ex(Z_ARRVAL_P(zval_list), &hp);
		SUCCESS == zend_hash_get_current_data_ex(Z_ARRVAL_P(zval_list), (void **) &item, &hp);
		i++, zend_hash_move_forward_ex(Z_ARRVAL_P(zval_list), &hp)
	) {
		/* ѿФʤΤǺzvalʤ */
		/* SEPARATE_ZVAL(item); */
		convert_to_string_ex(item);
		tag_list->len_list[i] = (unsigned int) Z_STRLEN_PP(item);
		tag_list->str_list[i] = estrndup(Z_STRVAL_PP(item), Z_STRLEN_PP(item));
		add_next_index_stringl(bank, tag_list->str_list[i], tag_list->len_list[i], 0); /* estrndupǳݤѿݻŪʤΤ0ꤹ */
	}
	
	return tag_list;
}

static int senna_snippet_parse_tag_list(INTERNAL_FUNCTION_PARAMETERS,
			char **query_string, int *query_string_len, long *max_exprs,
			long *encoding, long *flags, long *width, long *max_results, zval **zval_mapping, senna_snippet_object *this) {
	zval *tmpOpen, *argOpen = NULL;
	zval *tmpClose, *argClose = NULL;
	
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
				"sllll|lzzz",
				query_string, query_string_len, max_exprs, encoding, flags, width,
				max_results, &argOpen, &argClose, zval_mapping)) {
		return FAILURE;
	}

	/*
	 * argOpen򤽤Τޤconvert_to_array_ex()ƤӽФμ°Ȥޤ
	 * ѤäƤޤäġ͸ƤӽФˤʤäߤΤǰѿ롣
	 */
	
	/*
	 * SEPARATE_ZVAL()ʬΥ꤬ץȽλޤǲǤʤ餷
	 * ѤѿݤơȤȤˤ
	 */
	MAKE_STD_ZVAL(tmpOpen);
	MAKE_STD_ZVAL(tmpClose);
	
	if (argOpen) {
		*tmpOpen = *argOpen;
		zval_copy_ctor(tmpOpen);
	} else {
		ZVAL_NULL(tmpOpen);
	}
	
	if (argClose) {
		*tmpClose = *argClose;
		zval_copy_ctor(tmpClose);
	} else {
		ZVAL_NULL(tmpClose);
	}
	
	switch (Z_TYPE_P(tmpOpen)) {
		case IS_NULL:
		case IS_STRING:
		case IS_LONG:
		case IS_OBJECT:
			convert_to_string_ex(&tmpOpen);
			break;
		case IS_ARRAY:
			break;
		default:
			goto failure;
	}
	
	switch (Z_TYPE_P(tmpClose)) {
		case IS_NULL:
		case IS_STRING:
		case IS_LONG:
		case IS_OBJECT:
			convert_to_string_ex(&tmpClose);
			break;
		case IS_ARRAY:
			break;
		default:
			goto failure;
	}

	convert_to_array_ex(&tmpOpen);
	convert_to_array_ex(&tmpClose);
	
	/* ꥹȤ */
	this->open_tag_list = senna_snippet_create_tag_list(tmpOpen, this->tag_list);
	this->close_tag_list = senna_snippet_create_tag_list(tmpClose, this->tag_list);
	
	zval_ptr_dtor(&tmpOpen);
	zval_ptr_dtor(&tmpClose);
	return SUCCESS;
failure:
	zval_ptr_dtor(&tmpOpen);
	zval_ptr_dtor(&tmpClose);
	zend_throw_exception_ex(SPL_CE(InvalidArgumentException), 0 TSRMLS_CC,
			"Both argument #7(open tag) and #8(close tag) must either be a STRING or an ARRAY of STRINGs with the same number of elements");
	return FAILURE;
}

static int senna_snippet_validate_length(const char *keyword, unsigned int keyword_len, void *query_term_args) {
	query_term_args_t *args = (query_term_args_t *) query_term_args;
	unsigned int offset = args->query_len - strlen(keyword);
	int i;
	
	if (args->width < keyword_len) {
		for (i = 0; i < keyword_len; i++) {
			args->query[offset + i] = '\x20';
		}
		/* widthĶĹΥɤäȡ
		 * Υɤ˱黻ҤդƤ
		 * Υɤ˳ݤäƤޤǽ롣
		 *
		 * -widthĹʸ ̤Υɡ
		 *  -            ̤Υ פˤʤꡢ 
		 * ޥʥ黻Ҥ̤ΥɡפФƳݤ(֤)
		 * *פ֤Ȥǡ-* ̤Υɡפˤʤꡢ
		 * 黻Ҥ̵Ǥ롣
		 */
		args->query[offset + 0] = SEN_QUERY_PREFIX;
		args->changed = 1;
	}
	return 1;
}

/* {{{ proto Senna_Snippet Senna_Snippet::createQuerySnippet(string query_string, int max_exprs = Senna_Snippet::DEFAULT_MAX_N_EXPRS, int encoding, int flags, int width [, int max_results = 1 [, mixed openTag = '' [, mixed closeTag = '' [, mixed mapping = Senna_Snippet::MAPPING_NONE]]]])
   query_stringǻꤷʸ󤫤饤󥹥󥹤롣
   openTagcloseTagξ硢createQuerySnippet()ϥγƥɤȤ
   󤫤֤ͤҤȤĤļФΥѤΥȤƻѤ롣
   ǿɿ⾯ʤϡƬǤäƺƤӽ֤˻Ѥ롣
   (ΤᡢopenTagcloseTagǿƱǤʤФʤʤ)
 */
PHP_METHOD(Senna_Snippet, createQuerySnippet) {
	char *query_string;
	int query_string_len;
	long max_exprs;
	sen_encoding encoding;
	long flags;
	long width;
	long max_results = 1;
	zval *zval_mapping = NULL;
	
	sen_snip_mapping *mapping;
	senna_snippet_object *this;
	
	object_init_ex(return_value, SENNA_CE(Senna_Snippet));
	this = (senna_snippet_object *) zend_object_store_get_object(return_value TSRMLS_CC);

	php_set_error_handling(EH_THROW, SPL_CE(InvalidArgumentException) TSRMLS_CC);
	
	if (FAILURE == senna_snippet_parse_tag_list(INTERNAL_FUNCTION_PARAM_PASSTHRU,
				&query_string, &query_string_len,
				&max_exprs, (long *) &encoding, &flags, &width, &max_results, &zval_mapping, this)) {
		php_std_error_handling();
		return;
	}
	
	php_std_error_handling();
	
	if (FAILURE == senna_snippet_validate_constructor_args(3, encoding, flags, width, max_results, zval_mapping)) {
		return;
	}
	
	if (!query_string_len) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC, "Argument #1(query string) cannot be empty");
		return;
	}
	
	if ((max_exprs < 1) || (MAX_SNIP_COND_COUNT < max_exprs)) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
				"Argument #2(max expressions) must be 1..%d(Senna_Snippet::MAX_KEYWORD_COUNT), %d given", MAX_SNIP_COND_COUNT, max_exprs);
		return;
	}
	

	if (!zval_mapping || (IS_NULL == Z_TYPE_P(zval_mapping))) {
		mapping = (sen_snip_mapping *) NULL;
	} else {
		mapping = (sen_snip_mapping *) Z_LVAL_P(zval_mapping);
	}

	/* Ʊå */
	if (this->open_tag_list->count != this->close_tag_list->count) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
				"Both argument #7(open tag) and #8(close tag) must have an equal number of elements. "
				"count(arg#7) == %d, count(arg#8) == %d.", this->open_tag_list->count, this->close_tag_list->count);
		return;
	}
	
#if PHP_SENNA_0
	/* ǿ1ʾ夫å */
	if (!this->open_tag_list->count) {
		zend_throw_exception_ex(SPL_CE(UnexpectedValueException), 0 TSRMLS_CC,
				"Either argument #7(open tag) or #8(close tag) must not be empty");
		return;
	}
#endif

	/* ʸ󤫤Snippet֥Ȥ */
	{
		sen_query *q = sen_query_open(query_string, query_string_len, sen_sel_and, max_exprs, encoding);
		
		if (0 <= php_version_compare(SENNA_G(api_version), "1.1.0")) {
			/*
			 * Senna1.1.0sen_query_term()Х줿ΤǡƥɤĹåǽ
			 * [Senna-dev 716] http://lists.sourceforge.jp/mailman/archives/senna-dev/2007-November/000715.html
			 */
			/*
			 * widthʾĹΥɤ롣
			 *
			 * []
			 * ⤷widthʾĹΥɤޤޤƤȡsen_query_snip()ϼԤNULL֤
			 * query_stringϡץꥱѼ¦Ϥ¿Ϥʪ顢
			 * տޤĹʸϤȤˤ㳰ȯƤȤˤȻפ
			 *
			 * PHP¦ĹɤꤹΤϼ֤ꤹ롣
			 * ޤtry-catch񤤤ȤнǤޤ̣ʤ
			 * Ĺ륭ɼΤäƤޤɤ
			 * äɤΰϼǤʤ
			 */
			query_term_args_t args;
			args.width = width;
			args.changed = 0;
			args.query = estrndup(query_string, query_string_len);
			args.query_len = query_string_len;
			sen_query_term(q, senna_snippet_validate_length, (void *) &args);
			if (args.changed) {
				/* Ĺ륭ɤä硢äʸsen_queryľ */
				q = sen_query_open(args.query, args.query_len, sen_sel_and, max_exprs, encoding);
			}
			efree(args.query);
		}
		
		this->snip = sen_query_snip(
			q,
			(int) flags,
			(unsigned int) width,
			(unsigned int) max_results,
			(unsigned int) this->open_tag_list->count,
			(const char **) this->open_tag_list->str_list, this->open_tag_list->len_list,
			(const char **) this->close_tag_list->str_list, this->close_tag_list->len_list,
			mapping
		);
		sen_query_close(q);
		
		if (NULL == this->snip) {
			/* NULL֤Τϡ­λʤΤsen_memory_exhausted򥻥åȤ(max_resultsϥåѤߤΤʤϤ) */
			zend_throw_exception_ex(SENNA_CE(SennaException), sen_memory_exhausted TSRMLS_CC, "sen_query_snip() returned NULL");
			return;
		}
		
		this->width = width;
		this->keyword_count = max_exprs;  /* ɿ׻ǤʤΤǾ¿ƱͤꤷƤޤ */
	}
}

zend_function_entry senna_snippet_functions[] = {
	PHP_ME(Senna_Snippet, __construct,     NULL, ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
	PHP_ME(Senna_Snippet, addKeyword,      NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Snippet, getKeywordCount, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(Senna_Snippet, execute,         NULL, ZEND_ACC_PUBLIC)
	
	/* factory */
	PHP_ME(Senna_Snippet, createQuerySnippet, NULL, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
	
	/*
	snippetϥɤΥդȤϰ㤦ΤǤפʤ
	PHP_ME(Senna_Snippet, addKeywordCallback,      NULL, ZEND_ACC_PUBLIC) */

	{NULL, NULL, NULL}
};


PHP_MINIT_FUNCTION(senna_snippet) {
	REGISTER_SENNA_CLASS(Senna_Snippet, senna_snippet_functions, senna_snippet_new);

	REGISTER_SENNA_LONG_CONSTANT(Senna_Snippet, "NORMALIZE", SEN_SNIP_NORMALIZE);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Snippet, "SKIP_LEADING_SPACES", SEN_SNIP_SKIP_LEADING_SPACES); /* senna-1.1.2 */
	
	REGISTER_SENNA_LONG_CONSTANT(Senna_Snippet, "MAPPING_NONE",   PHP_SENNA_SNIPPET_MAPPING_NONE);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Snippet, "MAPPING_COMPAT", PHP_SENNA_SNIPPET_MAPPING_COMPAT);
	
	REGISTER_SENNA_LONG_CONSTANT(Senna_Snippet, "MAX_RESULT_COUNT",  MAX_SNIP_RESULT_COUNT);
	REGISTER_SENNA_LONG_CONSTANT(Senna_Snippet, "MAX_KEYWORD_COUNT", MAX_SNIP_COND_COUNT);
	
	REGISTER_SENNA_LONG_CONSTANT(Senna_Snippet, "DEFAULT_MAX_N_EXPRS", PHP_SENNA_DEFAULT_MAX_N_EXPRS);


	return SUCCESS;
}


/* The previous line is meant for vim and emacs, so it can correctly fold and 
   unfold functions in source code. See the corresponding marks just before 
   function definition, where the functions purpose is also documented. Please 
   follow this convention for the convenience of others editing your code.
*/


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
